import pytest

from opencve.commands.utils import CveUtil
from opencve.extensions import db
from opencve.models import vendors
from opencve.models.alerts import Alert
from opencve.models.cve import Cve
from opencve.models.users import User
from opencve.models.events import Event
from opencve.models.vendors import Vendor
from opencve.models.products import Product
from opencve.tasks.alerts import filter_events, handle_alerts


@pytest.mark.parametrize(
    "types",
    [
        (["cvss"]),
        (["cvss", "summary"]),
        (["cvss", "summary", "references"]),
        (["cvss", "summary", "references", "cpes"]),
        (["cvss", "summary", "references", "cpes", "cwes"]),
    ],
)
def test_filter_events(app, create_user, create_cve, open_file, types):
    def create_events(cve):
        for t in ["summary", "cpes", "cwes", "cvss", "references"]:
            CveUtil.create_event(
                cve, open_file(f"modified_cves/CVE-2018-18074_{t}.json")[0], t, {}
            )

    user = create_user()
    cve = create_cve("CVE-2018-18074")
    create_events(cve)

    events = Event.query.all()
    assert sorted([e.type.code for e in events]) == sorted(
        ["summary", "cpes", "cwes", "cvss", "references"]
    )

    assert sorted([e.type.code for e in filter_events(user, events)]) == sorted(
        ["summary", "cpes", "cwes", "cvss", "references"]
    )

    # Return events based on user's filters
    user.filters_notifications = {"cvss": 0, "event_types": types}
    db.session.commit()
    assert sorted([e.type.code for e in filter_events(user, events)]) == sorted(types)


def test_filter_events_first_time(create_user, create_cve, create_vendor, open_file):
    cve = create_cve("CVE-2018-18074")
    event = CveUtil.create_event(
        cve,
        open_file(f"modified_cves/CVE-2018-18074_first_time_1.json")[0],
        "first_time",
        ["opencveio", "opencveio$PRODUCT$opencveio"],
    )

    # User1 doesn't have the first_time type
    user1 = create_user("user1")
    user1.filters_notifications = {"cvss": 0, "event_types": ["summary"]}
    db.session.commit()
    assert filter_events(user1, [event]) == []

    # User2 hasn't subscribed to opencve vendor
    user2 = create_user("user2")
    vendor = create_vendor("not_existing")
    user2.vendors.append(vendor)
    db.session.commit()
    assert filter_events(user2, [event]) == []

    # User3 has subscribed to opencve vendor
    user3 = create_user("user3")
    vendor = create_vendor("opencveio")
    user3.vendors.append(vendor)
    db.session.commit()
    assert filter_events(user3, [event]) == [event]


def test_no_alerts(create_cve):
    handle_alerts()
    assert Alert.query.count() == 0

    # Create a CVE without events
    create_cve("CVE-2018-18074")
    assert Alert.query.count() == 0


def test_handle_alerts_no_subscription(create_cve, handle_events):
    # Create a new CVE with a 'new_cve' event
    handle_events("modified_cves/CVE-2018-18074.json")
    event = Event.query.first()
    assert event.type == "new_cve"
    assert event.review == False

    handle_alerts()

    assert event.review == True
    assert Alert.query.count() == 0


def test_alert_vendor_subscription(create_cve, create_user, handle_events):
    handle_events("modified_cves/CVE-2018-18074.json")
    cve = Cve.query.first()
    event = Event.query.first()
    assert event.type == "new_cve"
    assert event.review == False

    # Create a user with a vendor subscription
    user = create_user()
    vendor = Vendor.query.filter_by(name="canonical").first()
    user.vendors.append(vendor)
    db.session.commit()

    handle_alerts()
    assert event.review == True

    # An alert has been created
    alerts = Alert.query.all()
    assert len(alerts) == 1
    alert = alerts[0]
    assert alert.user.id == user.id
    assert len(alert.events) == 1
    assert alert.events[0].id == event.id
    assert alert.cve.id == cve.id
    assert alert.notify == False
    assert alert.details == {
        "filters": ["new_cve"],
        "products": [],
        "vendors": ["canonical"],
    }


def test_alert_product_subscription(create_cve, create_user, handle_events):
    handle_events("modified_cves/CVE-2018-18074.json")
    cve = Cve.query.first()
    event = Event.query.first()
    assert event.type == "new_cve"
    assert event.review == False

    # Create a user with a product subscription
    user = create_user()
    product = Product.query.filter_by(name="ubuntu_linux").first()
    user.products.append(product)
    db.session.commit()

    handle_alerts()
    assert event.review == True

    # An alert has been created
    alerts = Alert.query.all()
    assert len(alerts) == 1
    alert = alerts[0]
    assert alert.user.id == user.id
    assert len(alert.events) == 1
    assert alert.events[0].id == event.id
    assert alert.cve.id == cve.id
    assert alert.notify == False
    assert alert.details == {
        "filters": ["new_cve"],
        "products": ["ubuntu_linux"],
        "vendors": [],
    }


def test_alert_cvss_filter(create_cve, create_user, handle_events):
    handle_events("modified_cves/CVE-2018-18074.json")

    # Set the CVSS score to 5.0
    cve = Cve.query.first()
    cve.cvss3 = 5.0
    db.session.commit()

    event = Event.query.first()
    assert event.type == "new_cve"
    assert event.review == False

    # User1 will be alerted (5 > 1)
    vendor = Vendor.query.filter_by(name="canonical").first()
    user1 = create_user("user1")
    user1.vendors.append(vendor)
    user1.filters_notifications = {"cvss": 1.0, "event_types": ["new_cve"]}

    # User2 will not be alerted (5 < 6)
    user2 = create_user("user2")
    user2.vendors.append(vendor)
    user2.filters_notifications = {"cvss": 6.0, "event_types": ["new_cve"]}
    db.session.commit()

    handle_alerts()
    assert event.review == True

    # Only 1 alert has been created for user1
    alerts = Alert.query.all()
    assert len(alerts) == 1
    alert = alerts[0]
    assert alert.user.id == user1.id
    assert alert.notify == False
    assert alert.details == {
        "filters": ["new_cve"],
        "products": [],
        "vendors": ["canonical"],
    }


def test_alert_first_time_filter(create_cve, create_user, create_vendor, handle_events):
    create_cve("CVE-2018-18074")
    handle_events("modified_cves/CVE-2018-18074_first_time_1.json")

    # User1 has subscribed to opencve vendor
    user1 = create_user("user1")
    vendor = Vendor.query.filter_by(name="opencveio").first()
    user1.vendors.append(vendor)
    db.session.commit()

    # User2 hasn't subscribed to opencve vendor
    user2 = create_user("user2")
    vendor = create_vendor("another_vendor")
    user2.vendors.append(vendor)
    db.session.commit()

    handle_alerts()

    # There is only 1 alert for user1
    alert_1 = Alert.query.first()
    assert alert_1.user == user1
    assert sorted([e.type.code for e in alert_1.events]) == ["cpes", "first_time"]
    assert sorted(Event.query.filter_by(type="first_time").first().details) == [
        "opencveio",
        "opencveio$PRODUCT$opencve",
    ]

    # We trigger another version of this CVE, but there is no new
    # vendor/product, only a new version of an existing product
    handle_events("modified_cves/CVE-2018-18074_first_time_2.json")
    handle_alerts()

    # The new alert only contains the `cpes` event
    alerts = Alert.query.all()
    alerts.remove(alert_1)
    assert sorted([e.type.code for e in alerts[0].events]) == ["cpes"]


def test_alert_types_empty_filter(create_cve, create_user, handle_events):
    create_cve("CVE-2018-18074")
    vendor = Vendor.query.filter_by(name="canonical").first()

    handle_events("modified_cves/CVE-2018-18074_summary.json")
    event = Event.query.first()
    assert event.type == "summary"

    # The user doesn't want to be alerted for 'summary' events
    user = create_user("user1")
    user.vendors.append(vendor)
    user.filters_notifications = {"cvss": 1.0, "event_types": []}

    handle_alerts()
    assert event.review == True

    # No alert created
    alerts = Alert.query.all()
    assert len(alerts) == 0


def test_alert_types_filter(create_cve, create_user, handle_events):
    create_cve("CVE-2018-18074")
    vendor = Vendor.query.filter_by(name="canonical").first()

    handle_events("modified_cves/CVE-2018-18074_summary.json")
    event = Event.query.first()
    assert event.type == "summary"

    # User1 will be alerted (notification enabled for 'summary')
    vendor = Vendor.query.filter_by(name="canonical").first()
    user1 = create_user("user1")
    user1.vendors.append(vendor)
    user1.filters_notifications = {
        "cvss": 5.0,
        "event_types": ["new_cve", "references", "summary"],
    }

    # User2 will not be alerted (notification disabled for 'summary')
    user2 = create_user("user2")
    user2.vendors.append(vendor)
    user2.filters_notifications = {
        "cvss": 5.0,
        "event_types": ["new_cve", "cwes", "references"],
    }
    db.session.commit()

    handle_alerts()
    assert event.review == True

    # 1 alert created for user1
    alerts = Alert.query.all()
    assert len(alerts) == 1
    alert = alerts[0]
    assert alert.user.id == user1.id
    assert alert.notify == False
    assert alert.details == {
        "filters": ["summary"],
        "products": [],
        "vendors": ["canonical"],
    }



"""Add cascade deletions and indexes for reports and alerts

Revision ID: 1c7aecfc5f6e
Revises: 3f755e0484e0
Create Date: 2022-09-15 23:18:50.794977

"""

# revision identifiers, used by Alembic.
revision = "1c7aecfc5f6e"
down_revision = "3f755e0484e0"

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.drop_constraint("alerts_report_id_fkey", "alerts", type_="foreignkey")
    op.create_foreign_key(
        "alerts_report_id_fkey",
        "alerts",
        "reports",
        ["report_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.drop_constraint(
        "alerts_events_alert_id_fkey", "alerts_events", type_="foreignkey"
    )
    op.create_foreign_key(
        "alerts_events_alert_id_fkey",
        "alerts_events",
        "alerts",
        ["alert_id"],
        ["id"],
        ondelete="CASCADE",
    )

    op.create_index(op.f("ix_alerts_cve_id"), "alerts", ["cve_id"], unique=False)
    op.create_index(op.f("ix_alerts_user_id"), "alerts", ["user_id"], unique=False)
    op.create_index(op.f("ix_alerts_report_id"), "alerts", ["report_id"], unique=False)
    op.create_index(
        op.f("ix_alerts_events_alert_id"), "alerts_events", ["alert_id"], unique=False
    )


def downgrade():
    op.drop_constraint(
        "alerts_events_alert_id_fkey", "alerts_events", type_="foreignkey"
    )
    op.create_foreign_key(
        "alerts_events_alert_id_fkey", "alerts_events", "alerts", ["alert_id"], ["id"]
    )
    op.drop_constraint("alerts_report_id_fkey", "alerts", type_="foreignkey")
    op.create_foreign_key(
        "alerts_report_id_fkey", "alerts", "reports", ["report_id"], ["id"]
    )

    op.drop_index(op.f("ix_alerts_cve_id"), table_name="alerts")
    op.drop_index(op.f("ix_alerts_user_id"), table_name="alerts")
    op.drop_index(op.f("ix_alerts_report_id"), table_name="alerts")
    op.drop_index(op.f("ix_alerts_events_alert_id"), table_name="alerts_events")



from pathlib import Path

import click
from flask.cli import with_appcontext
from flask_migrate import upgrade

from opencve.commands import ensure_config


@click.command()
@ensure_config
@with_appcontext
def upgrade_db():
    """Create or upgrade the database."""
    migrations_path = Path(__file__).parent.parent.resolve() / "migrations"
    upgrade(directory=str(migrations_path))



from opencve.extensions import db
from opencve.models import BaseModel


class Cwe(BaseModel):
    __tablename__ = "cwes"

    cwe_id = db.Column(db.String(), nullable=False, index=True)
    name = db.Column(db.String())
    description = db.Column(db.String())

    @property
    def short_id(self):
        if not self.cwe_id.startswith("CWE-"):
            return None
        return self.cwe_id.split("CWE-")[1]

    def __repr__(self):
        return "<Cwe {}>".format(self.cwe_id)



from datetime import timedelta

import redis
from redis.lock import Lock
from celery import chain
from celery.schedules import crontab
from celery.utils.log import get_task_logger

from opencve.extensions import cel
from opencve.tasks.alerts import handle_alerts
from opencve.tasks.events import handle_events
from opencve.tasks.reports import handle_reports

logger = get_task_logger(__name__)


# Celery Beat configuration
CELERYBEAT_SCHEDULE = {}

# Periodic CVE check
CELERYBEAT_SCHEDULE["cve-updates-15-mn"] = {
    "task": "CVE_UPDATES",
    "schedule": timedelta(minutes=15),
}

# Periodic Reports cleanup
CELERYBEAT_SCHEDULE["reports-cleanup-daily"] = {
    "task": "REPORTS_CLEANUP",
    "schedule": crontab(minute=0, hour=0),
}


@cel.task(name="RELEASE_LOCK")
def release_lock():
    r = redis.Redis.from_url(cel.app.config.get("CELERY_LOCK_URL"))
    r.delete("cve_updates_lock")


@cel.task(name="CVE_UPDATES")
def cve_updates():
    cel.app.app_context().push()
    r = redis.Redis.from_url(cel.app.config.get("CELERY_LOCK_URL"))

    # The RELEASE_LOCK task release this lock after the chain completion or when
    # an error occurs, so we avoid parallel chains. A TTL of 12 hours is applied,
    # so the lock will be automatically released even if an unexpected problem occurs.
    acquired = Lock(r, "cve_updates_lock", timeout=3600 * 12, blocking=False).acquire()

    if not acquired:
        logger.info("Lock not acquired, skipping.")
        return False

    signature = chain(
        handle_events.si(), handle_alerts.si(), handle_reports.si(), release_lock.si()
    )
    signature.apply_async(link_error=release_lock.si())



from pathlib import Path
from unittest.mock import patch, Mock, PropertyMock

from opencve.extensions import db
from opencve.models.changes import Change
from opencve.models.events import Event
from opencve.models.tasks import Task
from opencve.models.cve import Cve
from opencve.models.metas import Meta
from opencve.tasks.events import has_changed, download_modified_items


@patch("opencve.tasks.events.requests.get")
def test_has_changed(mock_get, open_raw_file, handle_events):
    old = "default"
    new = "CE80026155D878F20E49576A1C7D2A52163CF4C52C1CCCC41497CA14E03761D5"
    type(mock_get.return_value).content = PropertyMock(
        return_value=open_raw_file("nvdcve-1.1-modified.meta").encode("utf-8")
    )

    # Create the initial meta
    db.session.add(Meta(name="nvd_last_sha256", value=old))
    db.session.commit()

    # The sha256 has changed
    meta, sha256 = has_changed()
    assert meta.value == old
    assert sha256 == new


@patch("opencve.tasks.events.requests.get")
def test_has_not_changed(mock_get, open_raw_file, handle_events):
    value = "CE80026155D878F20E49576A1C7D2A52163CF4C52C1CCCC41497CA14E03761D5"
    type(mock_get.return_value).content = PropertyMock(
        return_value=open_raw_file("nvdcve-1.1-modified.meta").encode("utf-8")
    )

    # Create the initial meta
    db.session.add(Meta(name="nvd_last_sha256", value=value))
    db.session.commit()

    # The sha256 has changed
    meta, sha256 = has_changed()
    assert meta.value == value
    assert sha256 == None


@patch("opencve.tasks.events.requests.get")
def test_download_modified_items(mock_get, open_raw_file):
    with open(
        Path(__file__).parent.parent.resolve()
        / "data/modified_cves/nvdcve-1.1-modified.json.gz"
    ) as f:
        data = f.buffer.read()
    type(mock_get.return_value).content = PropertyMock(return_value=data)
    items = download_modified_items()
    assert len(items) == 1
    assert items[0]["cve"]["CVE_data_meta"]["ID"] == "CVE-2005-0366"


def test_new_cve_detected(handle_events):
    cves = Cve.query.all()
    assert len(cves) == 0

    handle_events("modified_cves/CVE-2018-18074.json")

    # The CVE is created
    cves = Cve.query.all()
    assert len(cves) == 1

    cve = cves[0]
    assert cve.cve_id == "CVE-2018-18074"

    # Task has been created
    tasks = Task.query.all()
    assert len(tasks) == 1
    task = tasks[0]

    # Change has been created
    changes = Change.query.all()
    assert len(changes) == 1
    change = changes[0]
    assert change.task.id == task.id
    assert change.json["cve"]["CVE_data_meta"]["ID"] == "CVE-2018-18074"

    # Event has been created
    events = Event.query.all()
    assert len(events) == 1
    event = events[0]
    assert event.type == "new_cve"
    assert event.details == {}
    assert event.review == False
    assert event.cve.cve_id == "CVE-2018-18074"
    assert event.change.id == change.id



from sqlalchemy_utils import UUIDType

from opencve.context import _humanize_filter
from opencve.extensions import db
from opencve.models import BaseModel, users_products


class Product(BaseModel):
    __tablename__ = "products"

    name = db.Column(db.String(), nullable=False, index=True)

    # Relationships
    vendor_id = db.Column(UUIDType(binary=False), db.ForeignKey("vendors.id"))
    vendor = db.relationship("Vendor", back_populates="products")
    users = db.relationship("User", secondary=users_products)

    @property
    def human_name(self):
        return _humanize_filter(self.name)

    def __repr__(self):
        return "<Product {}>".format(self.name)



from datetime import datetime, timezone
from unittest.mock import patch

from opencve.extensions import db
from opencve.models.alerts import Alert
from opencve.models.vendors import Vendor
from opencve.models.reports import Report
from opencve.tasks.alerts import handle_alerts
from opencve.tasks.reports import handle_reports


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_list_alerts_authentication(mock_send, client, create_user, handle_events):
    user = create_user("opencve")
    handle_events("modified_cves/CVE-2018-18074.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    report = Report.query.first()
    alert = Alert.query.first()

    response = client.get(f"/api/reports/{report.public_link}/alerts")
    assert response.status_code == 401
    response = client.get(f"/api/reports/{report.public_link}/alerts/{alert.id}")
    assert response.status_code == 401

    client.login("opencve")
    response = client.get(f"/api/reports/{report.public_link}/alerts")
    assert response.status_code == 200
    response = client.get(f"/api/reports/{report.public_link}/alerts")
    assert response.status_code == 200


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_list_alerts(mock_send, client, create_user, create_cve, handle_events):
    create_cve("CVE-2018-18074")
    handle_events("modified_cves/CVE-2018-18074_cvss.json")
    user = create_user("opencve")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    report = Report.query.first()
    report.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)
    alert = Alert.query.first()
    alert.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)

    response = client.login("opencve").get(f"/api/reports/{report.public_link}/alerts")
    assert response.json == [
        {
            "created_at": "2021-01-01T00:00:00Z",
            "cve": "CVE-2018-18074",
            "details": {"filters": ["cvss"], "products": [], "vendors": ["canonical"]},
            "id": str(alert.id),
        },
    ]


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_get_alert(mock_send, client, create_user, create_cve, handle_events):
    create_cve("CVE-2018-18074")
    handle_events("modified_cves/CVE-2018-18074_cvss.json")
    user = create_user("opencve")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    report = Report.query.first()
    report.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)
    alert = Alert.query.first()
    alert.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)

    response = client.login("opencve").get(
        f"/api/reports/{report.public_link}/alerts/{alert.id}"
    )
    assert response.json == [
        {
            "cve": "CVE-2018-18074",
            "type": "cvss",
            "details": {"old": {"v2": 5.0, "v3": 9.8}, "new": {"v2": 6.0, "v3": 10}},
        }
    ]



import json

from flask import request, jsonify
from flask_user import current_user, login_required

from opencve.controllers.main import main
from opencve.extensions import db
from opencve.models.products import Product
from opencve.models.vendors import Vendor
from opencve.models import is_valid_uuid

from werkzeug.exceptions import HTTPException


@main.route("/subscriptions", methods=["POST"])
@login_required
def subscribe_to_tag():
    def _bad_request(type, id):
        return (
            jsonify({"status": "error", "message": f"{type} {id} does not exist"}),
            400,
        )

    if not current_user.is_authenticated:
        return json.dumps({"status": "error", "message": "not allowed"})

    # Check the required fields
    if not request.form["obj"] or not request.form["id"]:
        return json.dumps({"status": "error", "message": "bad request"})

    if not request.form["action"] or request.form["action"] not in [
        "subscribe",
        "unsubscribe",
    ]:
        return json.dumps({"status": "error", "message": "bad request"})

    # Vendor
    if request.form["obj"] == "vendor":
        if not is_valid_uuid(request.form["id"]):
            return _bad_request(request.form["obj"], request.form["id"])
        vendor = Vendor.query.get(request.form["id"])
        if not vendor:
            return _bad_request(request.form["obj"], request.form["id"])

        # Subscribe
        if request.form["action"] == "subscribe":
            if vendor not in current_user.vendors:
                current_user.vendors.append(vendor)
                db.session.commit()

            return json.dumps({"status": "ok", "message": "vendor added"})

        # Unsubscribe
        if request.form["action"] == "unsubscribe":
            if vendor in current_user.vendors:
                current_user.vendors.remove(vendor)
                db.session.commit()

            return json.dumps({"status": "ok", "message": "vendor removed"})

    # Product
    elif request.form["obj"] == "product":
        if not is_valid_uuid(request.form["id"]):
            return _bad_request(request.form["obj"], request.form["id"])
        product = Product.query.get(request.form["id"])
        if not product:
            return _bad_request(request.form["obj"], request.form["id"])

        # Subscribe
        if request.form["action"] == "subscribe":
            if product not in current_user.products:
                current_user.products.append(product)
                db.session.commit()

            return json.dumps({"status": "ok", "message": "product added"})

        # Unsubscribe
        if request.form["action"] == "unsubscribe":
            if product in current_user.products:
                current_user.products.remove(product)
                db.session.commit()

            return json.dumps({"status": "ok", "message": "product removed"})

    return json.dumps({"status": "error", "message": "bad request"})



import datetime

import arrow
from flask import abort
from flask_admin import AdminIndexView, expose
from flask_admin.contrib.sqla import ModelView
from flask_user import current_user
from sqlalchemy import desc, func
from sqlalchemy.orm import joinedload
from wtforms import PasswordField, validators


class AuthModelView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated and current_user.admin

    def inaccessible_callback(self, name, **kwargs):
        abort(404)


class HomeView(AdminIndexView):
    def is_accessible(self):
        return current_user.is_authenticated and current_user.admin

    def inaccessible_callback(self, name, **kwargs):
        abort(404)

    @expose("/")
    def index(self):
        if (not current_user.is_authenticated) or (not current_user.admin):
            abort(404)

        # Import here to avoid circular dependencies
        from opencve.extensions import db
        from opencve.models import users_products, users_vendors
        from opencve.models.cve import Cve
        from opencve.models.products import Product
        from opencve.models.reports import Report
        from opencve.models.tasks import Task
        from opencve.models.users import User
        from opencve.models.vendors import Vendor

        # Numbers of users
        users = User.query.count()

        # Numbers of confirmed users
        confirmed_users = User.query.filter(User.email_confirmed_at.isnot(None)).count()

        # Numbers of CVEs
        cves = Cve.query.count()

        # Numbers of generated reports
        reports = Report.query.count()

        # Numbers of vendors
        vendors = Vendor.query.count()

        # Numbers of products
        products = Product.query.count()

        # Last task date
        task_date = "--"
        task = Task.query.order_by(Task.created_at.desc()).first()
        if task:
            task_date = task.created_at.strftime("%a, %d %b %Y %H:%M:%S")

        # Number of vendors per user
        user_vendors = (
            db.session.query(
                User.id,
                User.username,
                func.count(users_vendors.c.user_id).label("total"),
            )
            .join(users_vendors)
            .group_by(User.id, User.username)
            .order_by(desc("total"))
            .limit(10)
            .all()
        )

        # Number of products per user
        user_products = (
            db.session.query(
                User.id,
                User.username,
                func.count(users_products.c.user_id).label("total"),
            )
            .join(users_products)
            .group_by(User.id, User.username)
            .order_by(desc("total"))
            .limit(10)
            .all()
        )

        # Number of reports per user
        user_reports = (
            db.session.query(
                User.id, User.username, func.count(Report.user_id).label("total")
            )
            .join(Report)
            .group_by(User.id, User.username)
            .order_by(desc("total"))
            .limit(10)
            .all()
        )

        # Number of users per day
        users_by_day = (
            db.session.query(
                func.date_trunc("day", User.created_at), func.count(User.id)
            )
            .group_by(func.date_trunc("day", User.created_at))
            .order_by(func.date_trunc("day", User.created_at))
            .all()
        )
        days = {
            "day": [arrow.get(user[0]).strftime("%d/%m/%y") for user in users_by_day],
            "count": [user[1] for user in users_by_day],
        }

        # Keep the last week
        week = {"day": days["day"][-7::], "count": days["count"][-7::]}

        # Number of users per month
        users_by_month = (
            db.session.query(
                func.date_trunc("month", User.created_at), func.count(User.id)
            )
            .group_by(func.date_trunc("month", User.created_at))
            .order_by(func.date_trunc("month", User.created_at))
            .all()
        )
        months = {
            "month": [
                arrow.get(month[0]).strftime("%B %Y") for month in users_by_month
            ],
            "count": [month[1] for month in users_by_month],
        }

        return self.render(
            "admin/index.html",
            statistics={
                "Total users": users,
                "Confirmed users": confirmed_users,
                "Total CVEs": cves,
                "Total reports": reports,
                "Total vendors": vendors,
                "Total products": products,
                "Last task": task_date,
            },
            users={
                "vendors": user_vendors,
                "products": user_products,
                "reports": user_reports,
            },
            week=week,
            days=days,
            months=months,
        )

    @expose("/tasks")
    def tasks(self):
        from .extensions import db
        from .models.tasks import Task

        tasks = (
            db.session.query(Task.created_at, Task.id, func.count(Task.id))
            .join(Task.changes)
            .group_by(Task)
            .order_by(Task.created_at.desc())
            .all()
        )

        return self.render("admin/tasks.html", tasks=tasks)

    @expose("/tasks/<id>")
    def task(self, id):
        from .models.tasks import Task
        from .models.changes import Change

        task = Task.query.get(id)
        changes = (
            Change.query.options(joinedload("cve"))
            .options(joinedload("events"))
            .filter_by(task_id=id)
            .order_by(Change.created_at.desc())
            .all()
        )

        return self.render("admin/task.html", task=task, changes=changes)


class UserModelView(AuthModelView):
    page_size = 20
    create_modal = False
    edit_modal = False
    can_view_details = True
    column_filters = column_searchable_list = ["username", "email"]
    column_list = ("username", "email", "created_at", "is_confirmed")
    column_details_list = (
        "username",
        "email",
        "created_at",
        "updated_at",
        "email_confirmed_at",
        "enable_notifications",
        "filters_notifications",
        "frequency_notifications",
        "first_name",
        "last_name",
        "active",
        "admin",
        "vendors",
        "products",
    )
    column_formatters_detail = dict(
        vendors=lambda v, c, m, p: ", ".join([v.name for v in m.vendors]),
        products=lambda v, c, m, p: ", ".join([p.name for p in m.products]),
    )

    # The real `password` attribute is not displayed. Instead we use 2
    # password inputs : one for the create user form (required) and the
    # other for the edit form (optional).
    form_args = dict(email=dict(validators=[validators.Email()]))
    form_excluded_columns = "password"
    form_extra_fields = {
        "create_password": PasswordField("Password", [validators.DataRequired()]),
        "edit_password": PasswordField("Password"),
    }
    form_widget_args = {
        "edit_password": {
            "placeholder": "Don't fill this input to keep the password unchanged",
        }
    }
    form_create_rules = (
        "username",
        "email",
        "create_password",
        "first_name",
        "last_name",
        "active",
        "admin",
    )
    form_edit_rules = (
        "username",
        "email",
        "edit_password",
        "first_name",
        "last_name",
        "active",
        "admin",
    )

    def on_model_change(self, form, User, is_created):
        if is_created:
            User.set_password(form.create_password.data)
            User.email_confirmed_at = datetime.datetime.utcnow()
        else:
            if form.edit_password.data.strip():
                User.set_password(form.edit_password.data)


class CveModelView(AuthModelView):
    page_size = 20
    can_create = False
    can_edit = False
    can_delete = False
    can_view_details = True
    column_filters = ["cve_id", "summary", "cvss2", "cvss3", "updated_at"]
    column_searchable_list = ["cve_id", "summary", "cvss2", "cvss3"]
    column_list = ("cve_id", "updated_at", "cvss2", "cvss3")


class EventModelView(AuthModelView):
    page_size = 20
    can_create = False
    can_edit = False
    can_delete = False
    can_view_details = True
    column_filters = column_searchable_list = ["type", "created_at"]
    column_list = ("cve", "type", "created_at")


class VendorModelView(AuthModelView):
    page_size = 20
    create_modal = False
    edit_modal = False
    can_view_details = True
    column_filters = column_searchable_list = ["name"]
    column_list = ["name", "created_at"]
    column_details_list = ["name", "users", "created_at", "updated_at"]
    column_formatters_detail = dict(users=lambda v, c, m, p: m.users)


class ProductModelView(AuthModelView):
    page_size = 20
    create_modal = False
    edit_modal = False
    can_view_details = True
    column_filters = column_searchable_list = ["name"]
    column_list = ["name", "vendor", "created_at"]
    column_details_list = ["vendor", "name", "users", "created_at", "updated_at"]
    column_formatters_detail = dict(users=lambda v, c, m, p: m.users)



from opencve.checks import BaseCheck
from opencve.commands.utils import CveUtil
from opencve.extensions import db


class Summary(BaseCheck):
    def execute(self):
        summary = self.cve_json["cve"]["description"]["description_data"][0]["value"]

        # Check if the summary has changed
        if self.cve_obj.summary != summary:
            # Replace it in the CVE
            old = self.cve_obj.summary
            self.cve_obj.summary = summary
            db.session.commit()

            # Create a 'summary' event
            event = CveUtil.create_event(
                self.cve_obj,
                self.cve_json,
                "summary",
                {"old": old, "new": self.cve_obj.summary},
            )
            return event

        return None



from opencve.context import _humanize_filter
from opencve.extensions import db
from opencve.models import BaseModel, users_vendors


class Vendor(BaseModel):
    __tablename__ = "vendors"

    name = db.Column(db.String(), nullable=False, unique=True)

    # Relationships
    products = db.relationship("Product", back_populates="vendor")
    users = db.relationship("User", secondary=users_vendors)

    @property
    def human_name(self):
        return _humanize_filter(self.name)

    def __repr__(self):
        return "<Vendor {}>".format(self.name)



import gzip
from io import BytesIO
import xml.etree.ElementTree

import requests
from cpe import CPE

from opencve.commands import header, info, timed_operation
from opencve.extensions import db
from opencve.models import get_uuid
from opencve.models.products import Product
from opencve.models.vendors import Vendor

NVD_CPE_URL = (
    "https://nvd.nist.gov/feeds/xml/cpe/dictionary/official-cpe-dictionary_v2.3.xml.gz"
)


def get_slug(vendor, product=None):
    slug = vendor
    if product:
        slug += "-{}".format(product)
    return slug


def run(mappings):
    """
    Import the Vendors and Products list.
    """
    header("Importing CPE list...")

    # Download the XML file
    with timed_operation("Downloading {}...".format(NVD_CPE_URL)):
        resp = requests.get(NVD_CPE_URL).content

    # Parse the XML elements
    with timed_operation("Parsing XML elements..."):
        raw = gzip.GzipFile(fileobj=BytesIO(resp))
        del resp
        items = set()
        for _, elem in xml.etree.ElementTree.iterparse(raw):
            if elem.tag.endswith("cpe23-item"):
                items.add(elem.get("name"))
            elem.clear()
        del raw

    # Create the objects
    with timed_operation("Creating list of mappings..."):
        for item in items:
            obj = CPE(item)
            vendor = obj.get_vendor()[0]
            product = obj.get_product()[0]

            if vendor not in mappings["vendors"].keys():
                mappings["vendors"][vendor] = dict(id=get_uuid(), name=vendor)

            if get_slug(vendor, product) not in mappings["products"].keys():
                mappings["products"][get_slug(vendor, product)] = dict(
                    id=get_uuid(),
                    name=product,
                    vendor_id=mappings["vendors"][vendor]["id"],
                )
        del items

    # Insert the objects in database
    with timed_operation("Inserting Vendors and Products..."):
        db.session.bulk_insert_mappings(Vendor, mappings["vendors"].values())
        db.session.bulk_insert_mappings(Product, mappings["products"].values())
        db.session.commit()

    info(
        "{} vendors and {} products imported.".format(
            len(mappings["vendors"]), len(mappings["products"])
        )
    )
    del mappings



def test_list_products_authentication(client, create_user, create_vendor):
    create_user("opencve")
    create_vendor("vendor1", "product1")
    response = client.get("/api/vendors/vendor1/products")
    assert response.status_code == 401
    response = client.login("john").get("/api/vendors/vendor1/products")
    assert response.status_code == 401
    response = client.login("opencve").get("/api/vendors/vendor1/products")
    assert response.status_code == 200


def test_list_products(client, create_user, create_vendor):
    create_user("opencve")
    create_vendor("vendor1")
    response = client.login("opencve").get("/api/vendors/vendor1/products")
    assert response.status_code == 200
    assert response.json == []

    create_vendor("vendor1", "product1")
    create_vendor("vendor1", "product2")
    response = client.login("opencve").get("/api/vendors/vendor1/products")
    assert response.status_code == 200
    assert len(response.json) == 2
    assert response.json == [
        {"human_name": "Product1", "name": "product1"},
        {"human_name": "Product2", "name": "product2"},
    ]


def test_get_product_not_found(client, create_user):
    create_user("opencve")
    response = client.login("opencve").get("/api/vendors/vendor1/products")
    assert response.status_code == 404
    assert response.json == {"message": "Not found."}


def test_get_product(client, create_user, create_vendor):
    create_user("opencve")
    create_vendor("vendor1", "product1")
    create_vendor("vendor1", "product2")

    response = client.login("opencve").get("/api/vendors/vendor1/products/product1")
    assert response.status_code == 200
    assert response.json == {"human_name": "Product1", "name": "product1"}


def test_list_product_cves(client, create_user, create_cve):
    create_user("opencve")
    create_cve("CVE-2018-18074")

    response = client.login("opencve").get(
        "/api/vendors/canonical/products/ubuntu_linux/cve"
    )
    assert len(response.json) == 1

    assert response.json[0] == {
        "created_at": "2018-10-09T17:29:00Z",
        "id": "CVE-2018-18074",
        "summary": "The Requests package before 2.20.0 for Python sends an HTTP Authorization header to an http URI upon receiving a same-hostname https-to-http redirect, which makes it easier for remote attackers to discover credentials by sniffing the network.",
        "updated_at": "2019-10-03T00:03:00Z",
    }


def test_list_flat_products(client, create_user, create_vendor):
    create_user("opencve")
    response = client.login("opencve").get("/api/products")
    assert response.status_code == 200
    assert response.json == []

    create_vendor("vendor1", "product1")
    create_vendor("vendor1", "product2")
    response = client.login("opencve").get("/api/products")
    assert response.status_code == 200
    assert len(response.json) == 2
    assert response.json == [
        {
            "human_name": "Product1",
            "name": "product1",
            "vendor_human_name": "Vendor1",
            "vendor_name": "vendor1",
        },
        {
            "human_name": "Product2",
            "name": "product2",
            "vendor_human_name": "Vendor1",
            "vendor_name": "vendor1",
        },
    ]

    response = client.login("opencve").get("/api/products?search=2")
    assert response.status_code == 200
    assert len(response.json) == 1
    assert response.json == [
        {
            "human_name": "Product2",
            "name": "product2",
            "vendor_human_name": "Vendor1",
            "vendor_name": "vendor1",
        },
    ]



import datetime

import click
from flask import current_app as app
from flask.cli import with_appcontext
from sqlalchemy.exc import IntegrityError

from opencve.commands import ensure_config, error, info
from opencve.extensions import db
from opencve.models.users import User


@click.command()
@click.argument("username")
@click.argument("email")
@click.password_option()
@click.option("--admin", is_flag=True, help="Grant user as admin.")
@ensure_config
@with_appcontext
def create_user(username, email, password, admin):
    """Create a user or admin."""
    if User.query.filter_by(username=username).first():
        raise click.BadParameter(f"{username} already exists.", param_hint="username")
    if User.query.filter_by(email=email).first():
        raise click.BadParameter(f"{email} already exists.", param_hint="email")

    user = User(
        username=username,
        email=email,
        active=True,
        admin=admin,
        email_confirmed_at=datetime.datetime.utcnow(),
        password=app.user_manager.hash_password(password),
    )
    db.session.add(user)

    try:
        db.session.commit()
    except IntegrityError as e:
        error(e)
    else:
        info("User {} created.".format(username))



import uuid

from sqlalchemy_utils import UUIDType

from opencve.extensions import db


def get_uuid():
    return str(uuid.uuid4())


def is_valid_uuid(val):
    """Check if a given value is a valid UUID"""
    try:
        uuid.UUID(str(val))
    except ValueError:
        return False
    return True


class BaseModel(db.Model):
    __abstract__ = True

    id = db.Column(
        UUIDType(binary=False), primary_key=True, nullable=False, default=get_uuid
    )
    created_at = db.Column(
        db.DateTime(timezone=True), default=db.func.now(), nullable=False, index=True
    )
    updated_at = db.Column(
        db.DateTime(timezone=True),
        default=db.func.now(),
        onupdate=db.func.now(),
        nullable=False,
    )

    def to_dict(self, attrs):
        return {attr: str(getattr(self, attr)) for attr in attrs}

    def __repr__(self):
        return """<{} '{}'>""".format(self.__class__.__name__, self.id)


users_vendors = db.Table(
    "users_vendors",
    db.Column(
        "user_id", UUIDType(binary=False), db.ForeignKey("users.id"), primary_key=True
    ),
    db.Column(
        "vendor_id",
        UUIDType(binary=False),
        db.ForeignKey("vendors.id"),
        primary_key=True,
    ),
)

users_products = db.Table(
    "users_products",
    db.Column(
        "user_id", UUIDType(binary=False), db.ForeignKey("users.id"), primary_key=True
    ),
    db.Column(
        "product_id",
        UUIDType(binary=False),
        db.ForeignKey("products.id"),
        primary_key=True,
    ),
)

alerts_events = db.Table(
    "alerts_events",
    db.Column(
        "alert_id",
        UUIDType(binary=False),
        db.ForeignKey("alerts.id", ondelete="CASCADE"),
        primary_key=True,
        index=True,
    ),
    db.Column(
        "event_id", UUIDType(binary=False), db.ForeignKey("events.id"), primary_key=True
    ),
)



import pytest
from werkzeug.exceptions import NotFound
from flask_paginate import Pagination
from flask_sqlalchemy import Pagination as PaginationSQLAlchemy

from opencve.controllers.base import BaseController
from opencve.models.cve import Cve


class TestController(BaseController):
    model = Cve
    order = [Cve.created_at.desc()]
    per_page_param = "CVES_PER_PAGE"
    schema = {
        "str": {"type": str, "default": "foobar"},
        "int": {"type": int, "default": 1234},
    }


def test_parse_args():
    args = TestController.parse_args({})
    assert args == {"page": 1}
    args = TestController.parse_args({"page": 2})
    assert args == {"page": 2}
    args = TestController.parse_args({"undeclared_key": "value"})
    assert args == {"page": 1}
    args = TestController.parse_args({"str": "value"})
    assert args == {"page": 1, "str": "value"}
    args = TestController.parse_args({"str": 1})
    assert args == {"page": 1, "str": "1"}
    args = TestController.parse_args({"int": 1})
    assert args == {"page": 1, "int": 1}
    args = TestController.parse_args({"int": 1})
    assert args == {"page": 1, "int": 1}
    args = TestController.parse_args({"int": "not_integer"})
    assert args == {"page": 1, "int": 1234}
    args = TestController.parse_args({"page": 10, "str": "foo", "int": 100})
    assert args == {"page": 10, "str": "foo", "int": 100}


def test_get(create_cve):
    with pytest.raises(NotFound):
        TestController.get({"cve_id": "CVE-2018-18074"})

    create_cve("CVE-2018-18074")
    obj = TestController.get({"cve_id": "CVE-2018-18074"})
    assert isinstance(obj, Cve)
    assert obj.cve_id == "CVE-2018-18074"


def test_list(app, create_cve):
    with app.test_request_context():
        objects, metas, pagination = TestController.list()
        assert isinstance(objects, PaginationSQLAlchemy)
        assert isinstance(pagination, Pagination)
        assert objects.items == []
        assert metas == {}

    create_cve("CVE-2018-18074")
    create_cve("CVE-2020-35188")
    with app.test_request_context():
        objects, _, _ = TestController.list()
        assert len(objects.items) == 2
        assert sorted([cve.cve_id for cve in objects.items]) == [
            "CVE-2018-18074",
            "CVE-2020-35188",
        ]


def test_list_items(app, create_cve):
    create_cve("CVE-2018-18074")
    create_cve("CVE-2020-35188")
    with app.test_request_context():
        objects = TestController.list_items()
        assert len(objects) == 2
        assert sorted([cve.cve_id for cve in objects]) == [
            "CVE-2018-18074",
            "CVE-2020-35188",
        ]



import gzip
import json
import re
from io import BytesIO

import arrow
import requests
from celery.utils.log import get_task_logger

from opencve.checks import BaseCheck
from opencve.commands.utils import CveUtil
from opencve.extensions import cel, db
from opencve.models.cve import Cve
from opencve.models.metas import Meta
from opencve.models.tasks import Task

NVD_MODIFIED_URL = "https://nvd.nist.gov/feeds/json/cve/1.1/nvdcve-1.1-modified.json.gz"
NVD_MODIFIED_META_URL = (
    "https://nvd.nist.gov/feeds/json/cve/1.1/nvdcve-1.1-modified.meta"
)
logger = get_task_logger(__name__)


def has_changed():
    logger.info("Downloading {}...".format(NVD_MODIFIED_META_URL))
    resp = requests.get(NVD_MODIFIED_META_URL)
    buf = BytesIO(resp.content).read().decode("utf-8")

    matches = re.match(r".*sha256:(\w{64}).*", buf, re.DOTALL)
    nvd_sha256 = matches.group(1)
    last_nvd256 = Meta.query.filter_by(name="nvd_last_sha256").first()

    if nvd_sha256 != last_nvd256.value:
        logger.info(
            "Found different hashes (old:{}, new:{}).".format(
                last_nvd256.value, nvd_sha256
            )
        )
        return last_nvd256, nvd_sha256
    else:
        logger.info("DB is up to date.")
        return last_nvd256, None


def download_modified_items():
    logger.info("Downloading {}...".format(NVD_MODIFIED_URL))
    resp = requests.get(NVD_MODIFIED_URL).content
    raw = gzip.GzipFile(fileobj=BytesIO(resp)).read()
    items = json.loads(raw.decode("utf-8"))["CVE_Items"]
    return items


def check_for_update(cve_json, task):
    cve_id = cve_json["cve"]["CVE_data_meta"]["ID"]
    cve_obj = Cve.query.filter_by(cve_id=cve_id).first()
    events = []

    # A new CVE has been added
    if not cve_obj:
        cve_obj = CveUtil.create_cve(cve_json)
        logger.info("{} created (ID: {})".format(cve_id, cve_obj.id))
        events = [CveUtil.create_event(cve_obj, cve_json, "new_cve", {})]

    # Existing CVE has changed
    elif CveUtil.cve_has_changed(cve_obj, cve_json):
        logger.info("{} has changed, parsing it...".format(cve_obj.cve_id))

        events = []
        checks = BaseCheck.__subclasses__()

        # Loop on each kind of check
        for check in checks:
            c = check(cve_obj, cve_json)
            event = c.execute()

            if event:
                events.append(event)

        # Change the last updated date
        cve_obj.updated_at = arrow.get(cve_json["lastModifiedDate"]).datetime
        cve_obj.json = cve_json
        db.session.commit()

    # Create the change
    if events:
        CveUtil.create_change(cve_obj, cve_json, task, events)


@cel.task(name="HANDLE_EVENTS")
def handle_events():
    cel.app.app_context().push()

    logger.info("Checking for new events...")
    current_sum, new_sum = has_changed()
    if not new_sum:
        return

    # Retrieve the list of modified CVEs
    logger.info("Download modified CVEs...")
    items = download_modified_items()

    # Create the task containing the changes
    task = Task()
    db.session.add(task)

    logger.info("Checking {} CVEs...".format(len(items)))
    for item in items:
        check_for_update(item, task)

    logger.info("CVEs checked, updating meta hash...")
    current_sum.value = new_sum
    db.session.commit()
    logger.info("Done, new meta is {}.".format(new_sum))



from opencve.extensions import db
from opencve.models.users import User
from opencve.admin import UserModelView
import pytest


@pytest.mark.freeze_time
def test_on_model_change_create_form(app, create_user, freezer):
    user = create_user("opencve")
    object = UserModelView(User, db.session)
    form = UserModelView.get_create_form(object)
    form.create_password.data = "create_password"
    freezer.move_to("2021-07-13")
    object.on_model_change(form, user, True)
    db.session.commit()
    assert app.user_manager.verify_password("create_password", user.password) == True
    assert user.email_confirmed_at.strftime("%Y-%m-%d") == "2021-07-13"


def test_on_model_change_edit_form(app, create_user):
    user = create_user("app")
    object = UserModelView(User, db.session)
    form = UserModelView.get_edit_form(object)
    form.edit_password.data = "edit_password"
    object._on_model_change(form, user, False)
    db.session.commit()
    assert app.user_manager.verify_password("edit_password", user.password) == True



from flask import current_app as app
from flask import abort, flash, redirect, render_template, request, url_for
from flask_user import current_user
from sqlalchemy import and_
from sqlalchemy.dialects.postgresql import array
from sqlalchemy.orm import joinedload, aliased

from opencve.constants import PRODUCT_SEPARATOR
from opencve.controllers.main import main, welcome
from opencve.controllers.reports import ReportController
from opencve.extensions import db
from opencve.forms import ActivitiesViewForm
from opencve.models.changes import Change
from opencve.models.cve import Cve
from opencve.models.events import Event


@welcome.route("/welcome")
def index():
    if not app.config.get("DISPLAY_WELCOME", False):
        abort(404)
    return render_template("index.html")


@welcome.route("/terms")
def terms():
    if not app.config.get("DISPLAY_TERMS", False):
        abort(404)
    return render_template("terms.html")


@main.route("/", methods=["GET", "POST"])
def home():
    # Allow customization of the homepage
    if not current_user.is_authenticated:
        if app.config.get("DISPLAY_WELCOME", False):
            return redirect(url_for("welcome.index"))
        return redirect(url_for("main.cves"))

    # Form used to customize the activities view
    activities_view_form = ActivitiesViewForm(
        obj=current_user,
        view=current_user.settings["activities_view"],
    )

    if request.method == "POST":
        form_name = request.form["form-name"]
        if form_name == "activities_view_form" and activities_view_form.validate():
            new_settings = {
                **current_user.settings,
                "activities_view": activities_view_form.view.data,
            }
            current_user.settings = new_settings
            db.session.commit()

            flash("Your settings has been updated.", "success")
            return redirect(url_for("main.home"))

    # Handle the page parameter
    page = request.args.get("page", type=int, default=1)
    page = 1 if page < 1 else page
    per_page = app.config["ACTIVITIES_PER_PAGE"]

    # Only display the 5 last reports
    reports = ReportController.list_items({"user_id": current_user.id})[:5]

    # Build the query to fetch the last changes
    query = (
        Change.query.options(joinedload("cve"))
        .options(joinedload("events"))
        .filter(Change.cve_id == Cve.id)
        .filter(Change.events.any())
    )

    # Filter by subscriptions
    if current_user.settings["activities_view"] == "subscriptions":
        vendors = [v.name for v in current_user.vendors]
        vendors.extend(
            [
                f"{p.vendor.name}{PRODUCT_SEPARATOR}{p.name}"
                for p in current_user.products
            ]
        )
        if not vendors:
            vendors = [None]
        query = query.filter(Cve.vendors.has_any(array(vendors)))

    # List the paginated changes
    changes = (
        query.order_by(Change.created_at.desc())
        .limit(per_page)
        .offset((page - 1) * per_page)
        .all()
    )

    return render_template(
        "home.html",
        changes=changes,
        reports=reports,
        page=page,
        activities_view_form=activities_view_form,
    )



import pytest
from flask import request
from werkzeug.exceptions import NotFound

from opencve.controllers.cves import CveController
from opencve.extensions import db
from opencve.models.tags import CveTag, UserTag


def test_metas(app, create_cves):
    create_cves(["CVE-2018-18074", "CVE-2020-9392", "CVE-2020-26116", "CVE-2020-27781"])

    with app.test_request_context():
        cves, metas, _ = CveController.list()
        assert len(cves.items) == 4
        assert metas == {"vendor": None, "product": None, "tag": None}

        cves, metas, _ = CveController.list({"vendor": "python"})
        assert len(cves.items) == 1
        assert metas["vendor"].name == "python"

        cves, metas, _ = CveController.list(
            {"vendor": "redhat", "product": "ceph_storage"}
        )
        assert len(cves.items) == 1
        assert metas["vendor"].name == "redhat"
        assert metas["product"].name == "ceph_storage"


def test_list_cves(app, create_cves):
    create_cves(["CVE-2018-18074", "CVE-2020-9392", "CVE-2020-26116", "CVE-2020-27781"])

    with app.test_request_context():
        cves = CveController.list_items()

    assert len(cves) == 4
    assert sorted([cve.cve_id for cve in cves]) == [
        "CVE-2018-18074",
        "CVE-2020-26116",
        "CVE-2020-27781",
        "CVE-2020-9392",
    ]


def test_list_cves_paginated(app, create_cves):
    old = app.config["CVES_PER_PAGE"]
    app.config["CVES_PER_PAGE"] = 3

    create_cves(
        [
            "CVE-2018-18074",
            "CVE-2020-9392",
            "CVE-2020-26116",
            "CVE-2020-27781",
            "CVE-2019-17052",
        ]
    )

    with app.test_request_context():
        cves = CveController.list_items()
        assert sorted([cve.cve_id for cve in cves]) == [
            "CVE-2019-17052",
            "CVE-2020-26116",
            "CVE-2020-27781",
        ]

        cves = CveController.list_items({"page": 2})
        assert sorted([cve.cve_id for cve in cves]) == [
            "CVE-2018-18074",
            "CVE-2020-9392",
        ]

        with pytest.raises(NotFound):
            cves = CveController.list_items({"page": 3})

    app.config["CVES_PER_PAGE"] = old


@pytest.mark.parametrize(
    "args,result",
    [
        ({"search": "nonexistingkeyword"}, []),
        ({"search": "CRLF"}, ["CVE-2020-26116"]),
        ({"search": "crlf"}, ["CVE-2020-26116"]),
        ({"search": "http"}, ["CVE-2018-18074", "CVE-2020-26116"]),
        ({"search": "18074"}, ["CVE-2018-18074"]),
        ({"search": "2020"}, ["CVE-2020-26116", "CVE-2020-9392"]),
        ({"search": "python"}, ["CVE-2018-18074", "CVE-2020-26116"]),
        ({"search": "canonical"}, ["CVE-2018-18074"]),
    ],
)
def test_search_cves(app, create_cves, args, result):
    create_cves(["CVE-2018-18074", "CVE-2020-9392", "CVE-2020-26116"])

    with app.test_request_context():
        cves = CveController.list_items(args)
    assert sorted([cve.cve_id for cve in cves]) == result


@pytest.mark.parametrize(
    "args,result",
    [
        ({"cwe": "CWE-276"}, ["CVE-2020-9392"]),
        ({"cwe": "CWE-522"}, ["CVE-2018-18074", "CVE-2020-27781"]),
        ({"cwe": "CWE-1234"}, []),
    ],
)
def test_filtered_by_cwe(app, create_cves, args, result):
    create_cves(["CVE-2018-18074", "CVE-2020-9392", "CVE-2020-26116", "CVE-2020-27781"])

    with app.test_request_context():
        cves = CveController.list_items(args)
    assert sorted([cve.cve_id for cve in cves]) == result


@pytest.mark.parametrize(
    "args,result",
    [
        ({"cvss": "none"}, ["CVE-2020-35076"]),
        ({"cvss": "low"}, ["CVE-2019-17052"]),
        ({"cvss": "medium"}, ["CVE-2020-29660"]),
        ({"cvss": "high"}, ["CVE-2020-26116", "CVE-2020-9392"]),
        ({"cvss": "critical"}, ["CVE-2018-18074"]),
    ],
)
def test_filtered_by_cvss(app, create_cves, args, result):
    create_cves(
        [
            "CVE-2018-18074",
            "CVE-2020-9392",
            "CVE-2020-26116",
            "CVE-2020-29660",
            "CVE-2019-17052",
            "CVE-2020-35076",
        ]
    )

    with app.test_request_context():
        cves = CveController.list_items(args)
    assert sorted([cve.cve_id for cve in cves]) == result


@pytest.mark.parametrize(
    "args,result",
    [
        ({"vendor": "redhat", "product": "ceph_storage"}, ["CVE-2020-27781"]),
        (
            {"vendor": "linux", "product": "linux_kernel"},
            ["CVE-2019-17052", "CVE-2019-8075"],
        ),
        ({"vendor": "redhat"}, ["CVE-2020-27781"]),
        ({"vendor": "linux"}, ["CVE-2019-17052", "CVE-2019-8075"]),
    ],
)
def test_filtered_by_vendors_products(app, create_cves, args, result):
    create_cves(["CVE-2019-8075", "CVE-2019-17052", "CVE-2020-27781"])

    with app.test_request_context():
        cves = CveController.list_items(args)
    assert sorted([cve.cve_id for cve in cves]) == result


def test_vendors_products_not_found(app):
    with app.test_request_context():
        with pytest.raises(NotFound):
            CveController.list_items({"vendor": "foo"})
        with pytest.raises(NotFound):
            CveController.list_items({"vendor": "foo", "product": "bar"})


def test_filtered_by_tags(app, create_cve, create_user):
    cve_2018_18074 = create_cve("CVE-2018-18074")
    cve_2020_9392 = create_cve("CVE-2020-9392")
    cve_2020_26116 = create_cve("CVE-2020-26116")
    create_cve("CVE-2020-27781")

    user = create_user()
    user.tags = [
        UserTag(name="tag1", description="foo", color="#fff"),
        UserTag(name="tag2", description="foo", color="#fff"),
    ]

    db.session.add(CveTag(user_id=user.id, cve_id=cve_2018_18074.id, tags=["tag1"]))
    db.session.add(
        CveTag(user_id=user.id, cve_id=cve_2020_9392.id, tags=["tag1", "tag2"])
    )
    db.session.add(CveTag(user_id=user.id, cve_id=cve_2020_26116.id, tags=["tag2"]))
    db.session.commit()

    # Tag is not in user's list of tags
    with pytest.raises(NotFound):
        CveController.list_items({"user_id": user.id, "tag": "notfound"})
    with app.test_request_context():
        cves = CveController.list_items()
        assert sorted([cve.cve_id for cve in cves]) == [
            "CVE-2018-18074",
            "CVE-2020-26116",
            "CVE-2020-27781",
            "CVE-2020-9392",
        ]
        cves = CveController.list_items({"user_id": user.id, "tag": "tag1"})
        assert sorted([cve.cve_id for cve in cves]) == [
            "CVE-2018-18074",
            "CVE-2020-9392",
        ]
        cves = CveController.list_items({"user_id": user.id, "tag": "tag2"})
        assert sorted([cve.cve_id for cve in cves]) == [
            "CVE-2020-26116",
            "CVE-2020-9392",
        ]



from flask import Blueprint
from flask_restful import Api

from opencve.api.alerts import AlertListResource, AlertResource
from opencve.api.cves import CveListResource, CveResource
from opencve.api.cwes import CweListResource, CweResource, CweCveResource
from opencve.api.products import (
    FlatProductListResource,
    ProductListResource,
    ProductResource,
    ProductCveResource,
)
from opencve.api.reports import ReportListResource, ReportResource
from opencve.api.vendors import VendorListResource, VendorResource, VendorCveResource

from opencve.api.subscriptions import (
    SubscriptionListRessourceVendor,
    SubscriptionListRessourceProduct,
)


api_bp = Blueprint("api", __name__)
api = Api(api_bp)


# Routes
api.add_resource(CweListResource, "/cwe")
api.add_resource(CweResource, "/cwe/<string:id>")
api.add_resource(CweCveResource, "/cwe/<string:id>/cve")
api.add_resource(CveListResource, "/cve")
api.add_resource(CveResource, "/cve/<string:id>")
api.add_resource(ReportListResource, "/reports")
api.add_resource(SubscriptionListRessourceVendor, "/account/subscriptions/vendors")
api.add_resource(SubscriptionListRessourceProduct, "/account/subscriptions/products")
api.add_resource(ReportResource, "/reports/<string:link>")
api.add_resource(AlertListResource, "/reports/<string:link>/alerts")
api.add_resource(AlertResource, "/reports/<string:link>/alerts/<string:id>")
api.add_resource(FlatProductListResource, "/products")
api.add_resource(VendorListResource, "/vendors")
api.add_resource(VendorResource, "/vendors/<string:name>")
api.add_resource(VendorCveResource, "/vendors/<string:name>/cve")
api.add_resource(ProductListResource, "/vendors/<string:vendor>/products")
api.add_resource(ProductResource, "/vendors/<string:vendor>/products/<string:product>")
api.add_resource(
    ProductCveResource, "/vendors/<string:vendor>/products/<string:product>/cve"
)



from flask import request, render_template

from opencve.controllers.main import main

# from opencve.controllers.cwes import get_cwes
from opencve.controllers.cwes import CweController


@main.route("/cwe")
def cwes():
    objects, _, pagination = CweController.list(request.args)
    return render_template("cwes.html", cwes=objects, pagination=pagination)



from functools import wraps

from flask import request
from flask import current_app as app
from flask_restful import Resource, HTTPException

from opencve.extensions import limiter
from opencve.models.users import User


def auth_required(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        auth = request.authorization

        def error_message():
            return (
                {"message": "Authentication required."},
                401,
                {"WWW-Authenticate": 'Basic realm="Authentication Required"'},
            )

        # Auth not provided
        if not auth:
            return error_message()

        # User not found
        user = User.query.filter_by(username=auth.username).first()
        if not user:
            return error_message()

        # Bad credentials
        if not app.user_manager.verify_password(auth.password, user.password):
            return error_message()

        f = func(*args, **kwargs)

        return f

    return wrapper


class BaseResource(Resource):
    decorators = [
        limiter.shared_limit(
            scope="api",
            limit_value=lambda: app.config["RATELIMIT_VALUE"],
            key_func=lambda: request.authorization.username,
        ),
        auth_required,
    ]



import pytest
from werkzeug.exceptions import NotFound

from opencve.controllers.cwes import CweController


def test_metas(app):
    with app.test_request_context():
        _, metas, _ = CweController.list()
    assert metas == {}


def test_list(app, create_cwes):
    create_cwes(["CWE-1", "CWE-2"])

    with app.test_request_context():
        cwes = CweController.list_items()
    assert len(cwes) == 2
    assert sorted([c.cwe_id for c in cwes]) == ["CWE-1", "CWE-2"]


def test_list_paginated(app, create_cwes):
    old = app.config["CWES_PER_PAGE"]
    app.config["CWES_PER_PAGE"] = 3

    create_cwes(["CWE-1", "CWE-2", "CWE-3", "CWE-4"])

    with app.test_request_context():
        cwes = CweController.list_items()
        assert sorted([c.cwe_id for c in cwes]) == ["CWE-2", "CWE-3", "CWE-4"]
        cwes = CweController.list_items({"page": 2})
        assert sorted([c.cwe_id for c in cwes]) == ["CWE-1"]

    app.config["CWES_PER_PAGE"] = old


@pytest.mark.parametrize(
    "args,result",
    [
        ({"search": "nonexistingkeyword"}, []),
        ({"search": "cwe-1"}, ["CWE-1"]),
        ({"search": "CWE-1"}, ["CWE-1"]),
        ({"search": " cWe-1 "}, ["CWE-1"]),
        ({"search": "2"}, ["CWE-2"]),
        ({"search": "credentials"}, ["CWE-522"]),
    ],
)
def test_by_search(app, create_cwes, create_cwe, args, result):
    create_cwes(["CWE-1", "CWE-2"])
    create_cwe(
        "CWE-522",
        "Insufficiently Protected Credentials",
        "The product transmits...",
    )

    with app.test_request_context():
        cwes = CweController.list_items(args)
    assert sorted([c.cwe_id for c in cwes]) == result



from flask import abort

from opencve.controllers.base import BaseController
from opencve.extensions import db
from opencve.models.alerts import Alert
from opencve.models.reports import Report


class ReportController(BaseController):
    model = Report
    order = [Report.created_at.desc()]
    per_page_param = "REPORTS_PER_PAGE"
    schema = {
        "user_id": {"type": str},
    }

    @classmethod
    def build_query(cls, args):
        query = Report.query.filter_by(user_id=args.get("user_id"))
        return query, {}

    @classmethod
    def get(cls, filters):
        report = super(ReportController, cls).get(filters)

        # The report is now seen
        report.seen = True
        db.session.commit()

        return report



import datetime

import pytest
from flask import request

from opencve.extensions import db
from opencve.models.users import User


def _create_user(app):
    user = User(
        username="user",
        email="user@opencve.io",
        active=True,
        admin=False,
        email_confirmed_at=datetime.datetime.utcnow(),
        password=app.user_manager.hash_password("password"),
    )
    db.session.add(user)
    db.session.commit()


def test_empty_login(app, client):
    with client:
        response = client.post(
            "/login", data=dict(username="", password=""), follow_redirects=True
        )
        assert request.path == "/login"
        assert b"Username is required" in response.data
        assert b"Password is required" in response.data


@pytest.mark.parametrize(
    "user,password",
    [
        ("user", "password"),
        ("user@opencve.io", "password"),
    ],
)
def test_login_success(app, client, user, password):
    _create_user(app)
    with client:
        response = client.post(
            "/login", data=dict(username=user, password=password), follow_redirects=True
        )

        assert request.path == "/"
        assert b"You have signed in successfully." in response.data


@pytest.mark.parametrize(
    "user,password",
    [
        ("user", "bad_password"),
        ("bad_user", "password"),
        ("bad_user", "bad_password"),
    ],
)
def test_login_errors(app, client, user, password):
    with client:
        response = client.post(
            "/login", data=dict(username=user, password=password), follow_redirects=True
        )
        assert request.path == "/login"
        assert response.data.count(b"Incorrect Username/Email and/or Password") == 2


def test_logout(app, client):
    _create_user(app)
    client.post(
        "/login", data=dict(username="user", password="password"), follow_redirects=True
    )

    with client:
        response = client.post("/logout", follow_redirects=True)
        assert request.path == "/login"
        assert b"You have signed out successfully." in response.data



from flask import request

from opencve.extensions import db
from opencve.models.users import User
from opencve.models.vendors import Vendor


def test_home_cve_redirect(client):
    response = client.get("/")
    assert response.status_code == 302

    with client:
        response = client.get("/", follow_redirects=True)
        assert response.status_code == 200
        assert request.path == "/cve"


def test_home_welcome_redirect(app, client):
    app.config["DISPLAY_WELCOME"] = True
    response = client.get("/")
    assert response.status_code == 302

    with client:
        response = client.get("/", follow_redirects=True)
        assert response.status_code == 200
        assert request.path == "/welcome"
        assert b"<title>Welcome to OpenCVE</title>" in response.data
        assert b"<h1>Welcome to OpenCVE</h1>" in response.data


def test_home_terms(app, client):
    response = client.get("/terms")
    assert response.status_code == 404

    response = client.get("/register")
    assert b"Terms of Service" not in response.data

    app.config["DISPLAY_TERMS"] = True
    response = client.get("/terms")
    assert response.status_code == 200
    assert b"Terms of Service and Privacy Policy" in response.data
    response = client.get("/register")
    assert b"Terms of Service" in response.data


def test_include_analytics(app, client):
    content = b'<script src="https://analytics.example.com/script.js"></script>'

    response = client.get("/cve")
    assert content not in response.data
    response = client.get("/login")
    assert content not in response.data

    app.config["INCLUDE_ANALYTICS"] = True
    response = client.get("/cve")
    assert content in response.data
    response = client.get("/login")
    assert content in response.data

    app.config["INCLUDE_ANALYTICS"] = False


def test_no_activity(client, login):
    response = client.get("/")
    assert response.status_code == 200
    assert b"No changes available." in response.data

    user = User.query.first()
    user.settings = {"activities_view": "subscriptions"}
    db.session.commit()

    response = client.get("/")
    assert response.status_code == 200
    assert (
        b'You have to subscribe to <a href="/vendors">products and vendors</a> to see their last changes.'
        in response.data
    )


def test_subscription_without_changes(client, login):
    user = User.query.first()
    vendor = Vendor(name="opencve")
    db.session.add(vendor)
    user.vendors.append(vendor)
    db.session.commit()

    response = client.get("/")
    assert response.status_code == 200
    assert b"No changes available." in response.data


def test_activity_new_cve(client, login, handle_events):
    user = User.query.first()
    handle_events("modified_cves/CVE-2018-18074.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()

    response = client.get("/")
    assert response.status_code == 200
    assert (
        b'<a href="/cve/CVE-2018-18074">CVE-2018-18074</a> is a new CVE'
        in response.data
    )


def test_activity_cve_changed(client, login, create_cve, handle_events):
    create_cve("CVE-2018-18074")
    user = User.query.first()
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()

    response = client.get("/")
    assert response.status_code == 200
    assert b"No changes available." in response.data

    handle_events("modified_cves/CVE-2018-18074_references.json")
    response = client.get("/")
    assert response.status_code == 200
    assert (
        b'<a href="/cve/CVE-2018-18074">CVE-2018-18074</a> has changed' in response.data
    )
    assert (
        b"<strong>1</strong> changed, <strong>1</strong> added, <strong>7</strong> removed"
        in response.data
    )


def test_list_paginated(app, client, login, handle_events, make_soup):
    old = app.config["ACTIVITIES_PER_PAGE"]
    app.config["ACTIVITIES_PER_PAGE"] = 2

    user = User.query.first()
    for cve in ["CVE-2018-18074", "CVE-2019-17052", "CVE-2020-26116"]:
        handle_events(f"modified_cves/{cve}.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    user.vendors.append(Vendor.query.filter_by(name="linux").first())
    user.vendors.append(Vendor.query.filter_by(name="python").first())
    db.session.commit()

    # Changes are ordered using the lastModifiedDate field of each CVE
    response = client.get("/")
    assert response.status_code == 200

    # Page 1
    soup = make_soup(response.data)
    dates = [s.find("span").text for s in soup.find_all(attrs={"class": "time-label"})]
    assert "04 Jan 2021" in dates[0]
    assert "19 Nov 2020" in dates[1]
    dates = [
        s.find("a").text for s in soup.find_all(attrs={"class": "timeline-header"})
    ]
    assert "CVE-2019-17052" in dates[0]
    assert "CVE-2020-26116" in dates[1]

    # Page 2
    response = client.get("/?page=2")
    soup = make_soup(response.data)
    dates = [s.find("span").text for s in soup.find_all(attrs={"class": "time-label"})]
    assert "03 Oct 2019" in dates[0]
    dates = [
        s.find("a").text for s in soup.find_all(attrs={"class": "timeline-header"})
    ]
    assert "CVE-2018-18074" in dates[0]

    # Other pages
    response = client.get("/?page=3")
    assert response.status_code == 200
    assert b"No changes available." in response.data
    response = client.get("/?page=300")
    assert response.status_code == 200
    assert b"No changes available." in response.data

    app.config["ACTIVITIES_PER_PAGE"] = old


def test_list_all_or_subscriptions_activities(client, login, create_cve, handle_events):
    create_cve("CVE-2018-18074")
    handle_events("modified_cves/CVE-2018-18074_references.json")
    create_cve("CVE-2020-35188")
    handle_events("modified_cves/CVE-2020-35188_cpes.json")

    user = User.query.first()
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()

    # By default activities_view is 'all'
    response = client.get("/")
    assert response.status_code == 200
    assert b"CVE-2018-18074" in response.data
    assert b"CVE-2020-35188" in response.data

    user.settings = {"activities_view": "subscriptions"}
    db.session.commit()

    # With the 'subscriptions' view the user doesn't see all the changes
    response = client.get("/")
    assert response.status_code == 200
    assert b"CVE-2018-18074" in response.data
    assert b"CVE-2020-35188" not in response.data



"""Add indexes on cves.summary and cve.cve_id

Revision ID: 33cd640e1112
Revises: f81abceece3d
Create Date: 2021-05-04 22:42:40.167721

"""

# revision identifiers, used by Alembic.
revision = "33cd640e1112"
down_revision = "f81abceece3d"

from alembic import op
import sqlalchemy as sa


def upgrade():
    """
    This migration introduces a requirement: the ability to enable the pg_trgm
    extension (https://www.postgresql.org/docs/current/pgtrgm.html).
    From PostgreSQL 13 this module is considered as trusted, meaning it can be
    installed by non-superusers with the CREATE privilege.
    """
    op.execute(sa.text("CREATE EXTENSION IF NOT EXISTS pg_trgm;"))
    op.create_index(
        "ix_cves_summary",
        "cves",
        ["summary"],
        unique=False,
        postgresql_using="gin",
        postgresql_ops={"summary": "gin_trgm_ops"},
    )
    op.drop_index("ix_cves_cve_id", table_name="cves")
    op.create_index(
        "ix_cves_cve_id",
        "cves",
        ["cve_id"],
        unique=False,
        postgresql_using="gin",
        postgresql_ops={"cve_id": "gin_trgm_ops"},
    )


def downgrade():
    op.drop_index("ix_cves_summary", table_name="cves")
    op.drop_index("ix_cves_cve_id", table_name="cves")
    op.create_index("ix_cves_cve_id", "cves", ["cve_id"], unique=False)
    op.execute(sa.text("DROP EXTENSION IF EXISTS pg_trgm;"))



def test_list_no_vendors(client):
    response = client.get("/vendors")
    assert response.status_code == 200
    assert b"Vendors (0)" in response.data
    assert b"Products (0)" in response.data


def test_list_vendors(client, create_vendor):
    create_vendor("vendor1", "product1")
    create_vendor("vendor2", "product1")
    create_vendor("vendor2", "product2")

    response = client.get("/vendors")
    assert response.status_code == 200
    assert b"Vendor1" in response.data
    assert b"Vendor2" in response.data
    assert b"Product1" in response.data
    assert b"Product2" in response.data
    assert b"Vendors (2)" in response.data
    assert b"Products (3)" in response.data


def test_vendor_search(client, create_vendor):
    create_vendor("the_vendor_1", "product1")
    create_vendor("the_vendor_2", "the_product_1")
    create_vendor("the_vendor_2", "product2")

    response = client.get("/vendors")
    assert b"Search in vendors and products" in response.data

    response = client.get("/vendors?search=Vendor 1")
    assert response.status_code == 200
    assert b"Vendors (1)" in response.data
    assert b"Products (0)" in response.data

    response = client.get("/vendors?search=the")
    assert b"Vendors (2)" in response.data
    assert b"Products (1)" in response.data
    assert b"The Vendor 1" in response.data
    assert b"The Vendor 2" in response.data
    assert b"The Product 1" in response.data

    response = client.get("/vendors?search=foobar")
    assert b"Vendors (0)" in response.data
    assert b"Products (0)" in response.data


def test_vendor_not_found(client, create_vendor):
    create_vendor("vendor1", "product1")
    response = client.get("/vendors?vendor=vendor1")
    assert response.status_code == 200
    response = client.get("/vendors?vendor=foobar")
    assert response.status_code == 404


def test_list_products(client, create_vendor):
    create_vendor("vendor1", "product1")
    create_vendor("vendor2", "product2")
    create_vendor("vendor2", "product3")

    response = client.get("/vendors?vendor=vendor2")
    assert b"Vendor <strong>Vendor2</strong> selected" in response.data
    assert b"Products (2)" in response.data
    assert b"Product2" in response.data
    assert b"Product3" in response.data


def test_search_products(client, create_vendor):
    create_vendor("vendor1", "product1")
    create_vendor("vendor2", "product1")
    create_vendor("vendor2", "product2")

    response = client.get("/vendors?vendor=vendor2")
    assert b"Vendor <strong>Vendor2</strong> selected" in response.data
    assert b"Search in <strong>Vendor2</strong> vendor" in response.data

    response = client.get("/vendors?vendor=vendor2&search=1")
    assert b"Products (1)" in response.data
    assert b"Product1" in response.data
    assert b"Product2" not in response.data


def test_vendors_products_paginated(app, client, create_vendor):
    old_vendors_setting = app.config["VENDORS_PER_PAGE"]
    old_products_setting = app.config["PRODUCTS_PER_PAGE"]
    app.config["VENDORS_PER_PAGE"] = 2
    app.config["PRODUCTS_PER_PAGE"] = 2

    for i in range(5):
        create_vendor(f"vendor{i}", f"product{i}")

    response = client.get("/vendors")
    assert b"Vendors (5)" in response.data
    assert b"Vendor0" in response.data
    assert b"Vendor1" in response.data
    assert b"Vendor2" not in response.data
    assert b"Products (5)" in response.data
    assert b"Product0" in response.data
    assert b"Product1" in response.data
    assert b"Product2" not in response.data

    response = client.get("/vendors?page=2")
    assert b"Vendors (5)" in response.data
    assert b"Vendor2" in response.data
    assert b"Vendor3" in response.data
    assert b"Vendor4" not in response.data
    assert b"Products (5)" in response.data
    assert b"Product0" in response.data
    assert b"Product1" in response.data
    assert b"Product2" not in response.data

    response = client.get("/vendors?page=2&product_page=2")
    assert b"Vendors (5)" in response.data
    assert b"Vendor2" in response.data
    assert b"Vendor3" in response.data
    assert b"Vendor4" not in response.data
    assert b"Products (5)" in response.data
    assert b"Product2" in response.data
    assert b"Product3" in response.data
    assert b"Product4" not in response.data

    response = client.get("/vendors?page=100&product_page=2")
    assert response.status_code == 404

    response = client.get("/vendors?product_page=100")
    assert response.status_code == 404

    app.config["VENDORS_PER_PAGE"] = old_vendors_setting
    app.config["PRODUCTS_PER_PAGE"] = old_products_setting



from flask import request
from flask_restful import fields, marshal_with

from opencve.api.base import BaseResource
from opencve.api.cves import cves_fields
from opencve.api.fields import HumanizedNameField, ProductsListField
from opencve.controllers.cves import CveController
from opencve.controllers.vendors import VendorController


vendor_list_fields = {
    "name": fields.String(attribute="name"),
    "human_name": HumanizedNameField(attribute="name"),
}

vendor_fields = dict(
    vendor_list_fields, **{"products": ProductsListField(attribute="products")}
)


class VendorListResource(BaseResource):
    @marshal_with(vendor_list_fields)
    def get(self):
        return VendorController.list_items(request.args)


class VendorResource(BaseResource):
    @marshal_with(vendor_fields)
    def get(self, name):
        return VendorController.get({"name": name})


class VendorCveResource(BaseResource):
    @marshal_with(cves_fields)
    def get(self, name):
        VendorController.get({"name": name})
        return CveController.list_items({**request.args, "vendor": name})



from opencve.checks import BaseCheck
from opencve.commands import info
from opencve.commands.utils import CveUtil
from opencve.extensions import db
from opencve.models.cwe import Cwe


class Cwes(BaseCheck):
    def execute(self):
        old = self.cve_obj.cwes
        new = [
            c["value"]
            for c in self.cve_json["cve"]["problemtype"]["problemtype_data"][0][
                "description"
            ]
        ]

        payload = {
            "added": list(set(new) - set(old)),
            "removed": list(set(old) - set(new)),
        }

        # It's possible that a CVE links a CWE not yet defined in database.
        # In this case we'll save it in the `cwes` table and a periodic task
        # will populate later its name and description using the MITRE file.
        for cwe_id in payload["added"]:
            cwe = Cwe.query.filter_by(cwe_id=cwe_id).first()

            if not cwe:
                info(
                    f"{cwe_id} detected in {self.cve_obj.cve_id} but not existing in database, adding it..."
                )
                cwe = Cwe(cwe_id=cwe_id)
                db.session.add(cwe)
                db.session.commit()

        # If the list of CWE changed
        if payload["added"] or payload["removed"]:

            # Save the new list
            self.cve_obj.cwes = new
            db.session.commit()

            # Create the event
            event = CveUtil.create_event(self.cve_obj, self.cve_json, "cwes", payload)
            return event

        return None



from datetime import datetime, timezone
from unittest.mock import patch

from opencve.extensions import db
from opencve.models.alerts import Alert
from opencve.models.reports import Report
from opencve.models.vendors import Vendor
from opencve.tasks.alerts import handle_alerts
from opencve.tasks.reports import handle_reports


def test_list_reports_authentication(client, create_user):
    create_user("opencve")
    response = client.get("/api/reports")
    assert response.status_code == 401
    response = client.login("john").get("/api/reports")
    assert response.status_code == 401
    response = client.login("opencve").get("/api/reports")
    assert response.status_code == 200


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_list_reports(mock_send, client, create_user, handle_events):
    user = create_user("opencve")
    response = client.login("opencve").get("/api/reports")
    assert response.status_code == 200
    assert response.json == []

    handle_events("modified_cves/CVE-2018-18074.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    report = Report.query.first()
    report.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)

    response = client.login("opencve").get("/api/reports")
    assert response.status_code == 200
    assert len(response.json) == 1
    assert response.json[0] == {
        "id": report.public_link,
        "details": ["canonical"],
        "created_at": "2021-01-01T00:00:00Z",
    }
    db.session.commit()


def test_get_report_not_found(client, create_user):
    create_user("opencve")
    response = client.login("opencve").get("/api/reports/404")
    assert response.status_code == 404
    assert response.json == {"message": "Not found."}


@patch("opencve.tasks.reports.user_manager.email_manager.send_user_report")
def test_get_report(mock_send, client, create_user, handle_events):
    user = create_user("opencve")
    handle_events("modified_cves/CVE-2018-18074.json")
    user.vendors.append(Vendor.query.filter_by(name="canonical").first())
    db.session.commit()
    handle_alerts()
    handle_reports()

    report = Report.query.first()
    report.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)
    alert = Alert.query.first()
    alert.created_at = datetime(2021, 1, 1, tzinfo=timezone.utc)

    response = client.login("opencve").get("/api/reports")
    assert response.status_code == 200
    assert len(response.json) == 1
    assert response.json[0] == {
        "id": report.public_link,
        "details": ["canonical"],
        "created_at": "2021-01-01T00:00:00Z",
    }

    response = client.login("opencve").get(f"/api/reports/{report.public_link}")
    assert response.status_code == 200

    data = response.json
    alerts = data.pop("alerts")
    assert alerts == [
        {
            "id": str(alert.id),
            "created_at": "2021-01-01T00:00:00Z",
            "cve": "CVE-2018-18074",
            "details": {
                "products": [],
                "vendors": ["canonical"],
                "filters": ["new_cve"],
            },
        }
    ]
    assert data == {
        "id": report.public_link,
        "details": ["canonical"],
        "created_at": "2021-01-01T00:00:00Z",
    }
    db.session.commit()



from flask import abort, request
from flask_restful import fields, marshal_with

from opencve.api.base import BaseResource
from opencve.api.fields import CveVendorsField, DatetimeField
from opencve.controllers.cves import CveController


cves_fields = {
    "id": fields.String(attribute="cve_id"),
    "summary": fields.String(attribute="summary"),
    "created_at": DatetimeField(),
    "updated_at": DatetimeField(),
}

cve_fields = dict(
    cves_fields,
    **{
        "cvss": {
            "v2": fields.Float(attribute="cvss2"),
            "v3": fields.Float(attribute="cvss3"),
        },
        "vendors": CveVendorsField(attribute="json"),
        "cwes": fields.Raw(),
        "raw_nvd_data": fields.Raw(attribute="json"),
    }
)


class CveListResource(BaseResource):
    @marshal_with(cves_fields)
    def get(self):
        return CveController.list_items(request.args)


class CveResource(BaseResource):
    @marshal_with(cve_fields)
    def get(self, id):
        return CveController.get({"cve_id": id})



from opencve.extensions import db
from opencve.models import BaseModel


class Meta(BaseModel):
    __tablename__ = "metas"

    name = db.Column(db.String(), nullable=False)
    value = db.Column(db.String(), nullable=False)

    def __repr__(self):
        return "<Meta {}>".format(self.name)



from opencve.checks import BaseCheck
from opencve.commands.utils import CveUtil
from opencve.extensions import db
from opencve.utils import convert_cpes, flatten_vendors


class FirstTime(BaseCheck):
    def execute(self):
        old = flatten_vendors(convert_cpes(self.cve_obj.json["configurations"]))
        new = flatten_vendors(convert_cpes(self.cve_json["configurations"]))
        payload = list(set(new) - set(old))

        if payload:
            event = CveUtil.create_event(
                self.cve_obj, self.cve_json, "first_time", payload
            )
            return event

        return None



from pathlib import Path

from setuptools import find_packages, setup


with open(
    Path(__file__).parent.resolve() / "opencve" / "VERSION", encoding="utf-8"
) as ver:
    version = ver.readline().rstrip()

with open("requirements.txt", encoding="utf-8") as req:
    requirements = [r.rstrip() for r in req.readlines()]


dev_requirements = [
    "pytest==5.4.1",
    "pytest-cov==2.11.1",
    "pytest-freezegun==0.4.2",
    "black==20.8b1",
    "beautifulsoup4==4.9.3",
]
sendmail_requirements = ["Flask-Sendmail-ng==0.3"]


setup(
    name="opencve",
    version=version,
    author="Nicolas Crocfer",
    author_email="ncrocfer@gmail.com",
    description="CVE Alerting Platform",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/opencve/opencve",
    packages=find_packages(),
    install_requires=requirements,
    extras_require={"dev": dev_requirements, "sendmail": sendmail_requirements},
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: Web Environment",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    entry_points={"console_scripts": ["opencve=opencve.cli:cli"]},
    python_requires=">=3.7",
)



from celery import Celery
from flask_admin import Admin
from flask_debugtoolbar import DebugToolbarExtension
from flask_login import current_user
from flask_gravatar import Gravatar
from flask_migrate import Migrate
from flask_sqlalchemy import SQLAlchemy
from flask_user import UserManager, EmailManager
from flask_user.forms import EditUserProfileForm, RegisterForm, unique_email_validator
from flask_wtf import RecaptchaField
from flask_wtf.csrf import CSRFProtect
from flask_limiter import Limiter
from wtforms import validators, StringField


class CustomUserManager(UserManager):
    """
    Add custom properties in default Flask-User objects.
    """

    def customize(self, app):
        def _unique_email_validator(form, field):
            """
            Check if the new email is unique. Skip this step if the
            email is the same as the current one.
            """
            if field.data.lower() == current_user.email.lower():
                return
            unique_email_validator(form, field)

        # Add the email field and make first and last names as not required
        class CustomUserProfileForm(EditUserProfileForm):
            first_name = StringField("First name")
            last_name = StringField("Last name")
            email = StringField(
                "Email",
                validators=[
                    validators.DataRequired(),
                    validators.Email(),
                    _unique_email_validator,
                ],
            )

        self.EditUserProfileFormClass = CustomUserProfileForm

        # Add the reCaptcha
        if app.config.get("DISPLAY_RECAPTCHA"):

            class CustomRegisterForm(RegisterForm):
                recaptcha = RecaptchaField()

            self.RegisterFormClass = CustomRegisterForm

        # Allow emails to be send using sendmail
        if app.config.get("EMAIL_ADAPTER") == "sendmail":
            from flask_user.email_adapters import SendmailEmailAdapter

            self.email_adapter = SendmailEmailAdapter(app)


class CustomEmailManager(EmailManager):
    def send_user_report(self, user, **kwargs):
        """Send the 'user report' email."""
        self._render_and_send_email(
            user.email,
            user,
            "emails/report",
            **kwargs,
        )


class FlaskCelery(Celery):
    """
    Provide the init_app function.
    """

    def __init__(self, *args, **kwargs):
        super(FlaskCelery, self).__init__(*args, **kwargs)

        if "app" in kwargs:
            self.init_app(kwargs["app"])

    def init_app(self, app):
        self.app = app
        self.conf.update(app.config.get("CELERY_CONF", {}))


# Debug toolbar
debug_toolbar = DebugToolbarExtension()

# CSRF protection
csrf = CSRFProtect()

# SQLAlchemy
db = SQLAlchemy(session_options={"autoflush": False})

# Flask gravatar
gravatar = Gravatar(
    size=100,
    rating="g",
    default="retro",
    force_default=False,
    use_ssl=True,
    base_url=None,
)

# Flask migrate
migrate = Migrate()

# Flask-User
user_manager = CustomUserManager(None, None, None)

# Celery
cel = FlaskCelery("opencve", include=["opencve.tasks"])

# Flask Limiter
limiter = Limiter(key_func=lambda: "Remove the default warning")



import os
import tempfile
from pathlib import Path
import warnings

from flask_admin import Admin
from flask_admin.base import MenuLink
from werkzeug.middleware.proxy_fix import ProxyFix

from opencve.admin import (
    CveModelView,
    EventModelView,
    HomeView,
    UserModelView,
    VendorModelView,
    ProductModelView,
)
from opencve.configuration import config
from opencve.extensions import CustomEmailManager
from opencve.extensions import (
    cel,
    csrf,
    db,
    debug_toolbar,
    gravatar,
    limiter,
    migrate,
    user_manager,
)
from opencve.models.cve import Cve
from opencve.models.events import Event
from opencve.models.products import Product
from opencve.models.users import User
from opencve.models.vendors import Vendor
from opencve.tasks import CELERYBEAT_SCHEDULE


class Config(object):
    SECRET_KEY = config.get("core", "secret_key")
    USE_REVERSE_PROXY = config.getboolean("core", "use_reverse_proxy", fallback=False)

    DISPLAY_WELCOME = config.getboolean("core", "display_welcome", fallback=False)
    DISPLAY_TERMS = config.getboolean("core", "display_terms", fallback=False)
    INCLUDE_ANALYTICS = config.getboolean("core", "include_analytics", fallback=False)

    # Customize the server name
    if config.get("core", "server_name"):
        SERVER_NAME = config.get("core", "server_name")

    # Database
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    SQLALCHEMY_DATABASE_URI = config.get("core", "database_uri")

    # Celery
    CELERY_CONF = {
        "BROKER_URL": config.get(
            "core", "celery_broker_url", fallback="redis://127.0.0.1:6379/0"
        ),
        "CELERY_RESULT_BACKEND": config.get(
            "core", "celery_result_backend", fallback="redis://127.0.0.1:6379/1"
        ),
        "CELERYBEAT_SCHEDULE": CELERYBEAT_SCHEDULE,
        "CELERY_TASK_SERIALIZER": "json",
        "CELERY_RESULT_SERIALIZER": "json",
        "CELERY_ACCEPT_CONTENT": ["json"],
        "CELERYD_HIJACK_ROOT_LOGGER": False,
    }
    CELERY_LOCK_URL = config.get(
        "core", "celery_lock_url", fallback="redis://127.0.0.1:6379/2"
    )

    # Pagination
    CVES_PER_PAGE = config.getint("core", "cves_per_page", fallback=20)
    VENDORS_PER_PAGE = config.getint("core", "vendors_per_page", fallback=20)
    PRODUCTS_PER_PAGE = config.getint("core", "products_per_page", fallback=20)
    CWES_PER_PAGE = config.getint("core", "cwes_per_page", fallback=20)
    REPORTS_PER_PAGE = config.getint("core", "reports_per_page", fallback=20)
    ALERTS_PER_PAGE = config.getint("core", "alerts_per_page", fallback=20)
    TAGS_PER_PAGE = config.getint("core", "tags_per_page", fallback=20)
    ACTIVITIES_PER_PAGE = config.getint("core", "activities_per_page", fallback=20)

    # ReCaptcha
    DISPLAY_RECAPTCHA = config.getboolean("core", "display_recaptcha", fallback=False)
    RECAPTCHA_PUBLIC_KEY = config.get("core", "recaptcha_site_key")
    RECAPTCHA_PRIVATE_KEY = config.get("core", "recaptcha_secret_key")

    # Vendors and Products excerpt
    COUNT_EXCERPT = 3
    REPORT_COUNT_EXCERPT = 3

    # Cleanup
    REPORTS_CLEANUP_DAYS = config.getint("core", "reports_cleanup_days", fallback=0)

    # Flask-User
    USER_APP_NAME = "OpenCVE.io"
    USER_ENABLE_CHANGE_USERNAME = False
    USER_ENABLE_MULTIPLE_EMAILS = True
    USER_AUTO_LOGIN_AFTER_CONFIRM = False

    # Flask-User redirect pages
    USER_AFTER_CHANGE_PASSWORD_ENDPOINT = "user.login"
    USER_AFTER_REGISTER_ENDPOINT = "user.login"
    USER_AFTER_CONFIRM_ENDPOINT = "user.login"
    USER_AFTER_FORGOT_PASSWORD_ENDPOINT = "user.login"
    USER_AFTER_RESET_PASSWORD_ENDPOINT = "user.login"
    USER_AFTER_LOGOUT_ENDPOINT = "user.login"
    USER_AFTER_RESEND_EMAIL_CONFIRMATION_ENDPOINT = "user.login"
    USER_AFTER_EDIT_USER_PROFILE_ENDPOINT = "user.edit_user_profile"

    # Flask-User urls
    USER_LOGIN_URL = "/login"
    USER_LOGOUT_URL = "/logout"
    USER_REGISTER_URL = "/register"
    USER_CHANGE_PASSWORD_URL = "/account/password"
    USER_EDIT_USER_PROFILE_URL = "/account/profile"
    USER_FORGOT_PASSWORD_URL = "/account/forgot-password"
    USER_RESEND_EMAIL_CONFIRMATION_URL = "/account/resend-email-confirmation"

    # API rate limit
    RATELIMIT_ENABLED = config.getboolean("api", "ratelimit_enabled", fallback=False)
    RATELIMIT_VALUE = config.get("api", "ratelimit_value", fallback="3600/hour")
    RATELIMIT_STORAGE_URL = config.get(
        "api", "ratelimit_storage_url", fallback="redis://127.0.0.1:6379/2"
    )
    RATELIMIT_HEADERS_ENABLED = True

    # Mail
    EMAIL_ADAPTER = config.get("mail", "email_adapter", fallback="smtp")
    USER_EMAIL_SENDER_EMAIL = config.get(
        "mail", "email_from", fallback="no-reply@opencve.io"
    )
    MAIL_SERVER = config.get("mail", "smtp_server")
    MAIL_PORT = config.getint("mail", "smtp_port", fallback=465)
    MAIL_USE_TLS = config.getboolean("mail", "smtp_use_tls", fallback=True)
    MAIL_USE_SSL = config.getboolean("mail", "smtp_use_ssl", fallback=False)
    MAIL_USERNAME = config.get("mail", "smtp_username")

    # ensure compatibility before deprecating "stmp_password"
    # see https://github.com/opencve/opencve/issues/76
    try:
        MAIL_PASSWORD = config.get("mail", "smtp_password")
    except:
        MAIL_PASSWORD = config.get("mail", "stmp_password")

    DEFAULT_MAIL_SENDER = config.get(
        "mail", "email_from", fallback="no-reply@opencve.io"
    )

    @staticmethod
    def init_app(app):
        # Flask-DebugToolbar
        debug_toolbar.init_app(app)

        # Celery
        cel.init_app(app)

        # CSRF protection
        csrf.init_app(app)

        # SQLAlchemy
        db.init_app(app)

        # User
        user_manager.init_app(app, db, User)
        user_manager.email_manager = CustomEmailManager(app)

        # Flask-Gravatar
        gravatar.init_app(app)

        # Flask-Migrate
        migrate.init_app(
            app=app,
            db=db,
            directory=str(Path(__file__).resolve().parent / "migrations"),
        )

        # Reverse proxy should used HTTPS
        if app.config["USE_REVERSE_PROXY"]:
            app.config["PREFERRED_URL_SCHEME"] = "https"
            app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1, x_host=1)

        # API Ratelimit
        limiter.init_app(app)

        # Flask-Admin
        admin = Admin(
            name="OpenCVE Admin", template_mode="bootstrap3", index_view=HomeView()
        )
        admin.init_app(app)
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore", "Fields missing from ruleset")
            admin.add_view(UserModelView(User, db.session))
        admin.add_view(CveModelView(Cve, db.session))
        admin.add_view(EventModelView(Event, db.session))
        admin.add_view(VendorModelView(Vendor, db.session))
        admin.add_view(ProductModelView(Product, db.session))
        admin.add_link(MenuLink(name="Tasks", url="/admin/tasks"))


class ProdConfig(Config):
    ENV = "prod"
    DEBUG = False


class DevConfig(Config):
    ENV = "development"
    DEBUG = True
    DEBUG_TB_INTERCEPT_REDIRECTS = False

    @staticmethod
    def init_app(app):
        Config.init_app(app)


class TestConfig(Config):
    db_file = tempfile.NamedTemporaryFile()

    ENV = "test"
    WTF_CSRF_ENABLED = False
    DEBUG_TB_INTERCEPT_REDIRECTS = False

    @staticmethod
    def init_app(app):
        Config.init_app(app)


env_config = {"development": DevConfig, "test": TestConfig, "production": ProdConfig}



from sqlalchemy_utils import JSONType, UUIDType

from opencve.extensions import db
from opencve.models import BaseModel, alerts_events


class Alert(BaseModel):
    __tablename__ = "alerts"

    details = db.Column(JSONType)
    notify = db.Column(db.Boolean, default=False)

    # Relationships
    events = db.relationship("Event", secondary=alerts_events, cascade="all, delete")

    user_id = db.Column(UUIDType(binary=False), db.ForeignKey("users.id"), index=True)
    user = db.relationship("User", back_populates="alerts")

    cve_id = db.Column(UUIDType(binary=False), db.ForeignKey("cves.id"), index=True)
    cve = db.relationship("Cve", back_populates="alerts")

    report_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("reports.id", ondelete="CASCADE"),
        nullable=True,
        index=True,
    )
    report = db.relationship("Report", back_populates="alerts")

    def __repr__(self):
        return "<Alert {}>".format(self.id)



from sqlalchemy.dialects.postgresql import JSONB
from flask_user import current_user

from opencve.extensions import db
from opencve.models import BaseModel
from opencve.models.tags import CveTag, UserTag


class Cve(BaseModel):
    __tablename__ = "cves"

    # CVE are sorted by last modified date, we need to index it.
    updated_at = db.Column(
        db.DateTime(timezone=True),
        default=db.func.now(),
        onupdate=db.func.now(),
        nullable=False,
        index=True,
    )

    cve_id = db.Column(db.String(), nullable=False)
    json = db.Column(JSONB)

    # We used initially secondary relationships to fetch the list of
    # associated vendors, products and cwes. But it was complicated
    # to maintain, and the performance were poor. So we now use the
    # JSONB data type associated to the GIN index type.
    vendors = db.Column(JSONB)
    cwes = db.Column(JSONB)

    # Keep the summary separated when searching keywords
    summary = db.Column(db.String(), nullable=False)

    # Keep CVSS separated when searching a particupal score
    cvss2 = db.Column(db.Float())
    cvss3 = db.Column(db.Float())

    # Relationships
    events = db.relationship("Event", back_populates="cve")
    changes = db.relationship("Change", back_populates="cve")
    alerts = db.relationship("Alert", back_populates="cve")

    # Index
    __table_args__ = (
        db.Index("ix_cves_vendors", vendors, postgresql_using="gin"),
        db.Index("ix_cves_cwes", cwes, postgresql_using="gin"),
        db.Index(
            "ix_cves_summary",
            summary,
            postgresql_using="gin",
            postgresql_ops={
                "summary": "gin_trgm_ops",
            },
        ),
        db.Index(
            "ix_cves_cve_id",
            cve_id,
            postgresql_using="gin",
            postgresql_ops={
                "cve_id": "gin_trgm_ops",
            },
        ),
    )

    def __repr__(self):
        return "<Cve {}>".format(self.cve_id)

    @property
    def raw_tags(self):
        if not current_user.is_authenticated:
            return []

        cve_tag = CveTag.query.filter_by(
            user_id=current_user.id, cve_id=self.id
        ).first()
        if not cve_tag:
            return []

        return cve_tag.tags

    @property
    def tags(self):
        if not current_user.is_authenticated:
            return []
        return [
            UserTag.query.filter_by(user_id=current_user.id, name=t).first()
            for t in self.raw_tags
        ]

    @property
    def cvss_weight(self):
        """Only used to sort several CVE by their CVSS"""
        w = 0
        if self.cvss2:
            w += self.cvss2
        if self.cvss3:
            w += self.cvss3
        return w



from opencve.checks import BaseCheck
from opencve.commands.utils import CveUtil
from opencve.extensions import db


class Cvss(BaseCheck):
    def execute(self):
        # Some CVE does not have CVSS, or they just have one version
        # Check the old values
        old = {}
        if self.cve_obj.cvss2:
            old["v2"] = self.cve_obj.cvss2
        if self.cve_obj.cvss3:
            old["v3"] = self.cve_obj.cvss3

        # Check the new values
        new = {}
        if "baseMetricV2" in self.cve_json["impact"]:
            new["v2"] = self.cve_json["impact"]["baseMetricV2"]["cvssV2"]["baseScore"]
        if "baseMetricV3" in self.cve_json["impact"]:
            new["v3"] = self.cve_json["impact"]["baseMetricV3"]["cvssV3"]["baseScore"]

        # If at least one version has changed, update the CVE
        if old != new:
            self.cve_obj.cvss2 = new.get("v2")
            self.cve_obj.cvss3 = new.get("v3")
            db.session.commit()

            # Create the event with the CVSS changes
            event = CveUtil.create_event(
                self.cve_obj, self.cve_json, "cvss", {"old": old, "new": new}
            )
            return event

        return None



from flask import request
from flask_restful import fields, marshal_with

from opencve.api.alerts import alert_fields
from opencve.api.base import BaseResource
from opencve.api.cves import cves_fields
from opencve.api.fields import DatetimeField
from opencve.controllers.alerts import AlertController
from opencve.controllers.reports import ReportController
from opencve.models.users import User


report_list_fields = {
    "id": fields.String(attribute="public_link"),
    "created_at": DatetimeField(),
    "details": fields.Raw(attribute="details"),
}

report_fields = dict(
    report_list_fields, **{"alerts": fields.List(fields.Nested(alert_fields))}
)


class ReportListResource(BaseResource):
    @marshal_with(report_list_fields)
    def get(self):
        user = User.query.filter_by(
            username=request.authorization.get("username")
        ).first()
        return ReportController.list_items({**request.args, "user_id": user.id})


class ReportResource(BaseResource):
    @marshal_with(report_fields)
    def get(self, link):
        report = ReportController.get({"public_link": link})
        setattr(report, "alerts", AlertController.list_items({"report_id": report.id}))
        return report



from opencve.controllers.vendors import VendorController


def test_subscribe_to_vendor_invalid_action(login, client, create_vendor):

    test_vendor = create_vendor("test vendor", "test product")

    with client:
        response = client.post(
            "/subscriptions",
            data={
                "obj": "vendor",
                "id": test_vendor.id,
                "action": "non-existent action",
            },
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert b'"message": "bad request"' in response.data
        assert b'"status": "error"' in response.data


def test_subscribe_to_vendor(login, client, create_vendor):

    test_vendor = create_vendor("test vendor", "test product")

    with client:
        response = client.post(
            "/subscriptions",
            data={"obj": "vendor", "id": test_vendor.id, "action": "subscribe"},
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert b'"message": "vendor added"' in response.data
        assert b'"status": "ok"' in response.data


def test_subscribe_to_product(login, client, create_vendor):

    test_vendor = create_vendor("test vendor", "test product")

    with client:
        response = client.post(
            "/subscriptions",
            data={
                "obj": "product",
                "id": test_vendor.products[0].id,
                "action": "subscribe",
            },
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert b'"message": "product added"' in response.data
        assert b'"status": "ok"' in response.data


def test_unsubscribe_to_vendor_subscribed_to(login, client, create_vendor):

    test_vendor = create_vendor("test vendor", "test product")

    with client:
        # Subscribe to the vendor
        response = client.post(
            "/subscriptions",
            data={"obj": "vendor", "id": test_vendor.id, "action": "subscribe"},
            follow_redirects=True,
        )

        # Get the vendor subscribed to
        test_vendor = VendorController.list_items()[0]

        # Unsubscribe to the vendor
        response = client.post(
            "/subscriptions",
            data={"obj": "vendor", "id": test_vendor.id, "action": "unsubscribe"},
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert b'"message": "vendor removed"' in response.data
        assert b'"status": "ok"' in response.data


def test_unsubscribe_to_product_subscribed_to(login, client, create_vendor):

    test_vendor = create_vendor("test vendor", "test product")

    with client:
        response = client.post(
            "/subscriptions",
            data={
                "obj": "product",
                "id": test_vendor.products[0].id,
                "action": "subscribe",
            },
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert b'"message": "product added"' in response.data
        assert b'"status": "ok"' in response.data

        response = client.post(
            "/subscriptions",
            data={
                "obj": "product",
                "id": test_vendor.products[0].id,
                "action": "unsubscribe",
            },
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert b'"message": "product removed"' in response.data
        assert b'"status": "ok"' in response.data


def test_subscribe_to_invalid_vendor_id_with_invalid_uuid(login, client):

    with client:
        response = client.post(
            "/subscriptions",
            data={
                "obj": "vendor",
                "id": "invalid_id",
                "action": "subscribe",
            },
            follow_redirects=True,
        )

        assert response.status_code == 400
        assert response.json["message"] == "vendor invalid_id does not exist"
        assert response.json["status"] == "error"


def test_subscribe_to_invalid_product_id_with_invalid_uuid(login, client):

    with client:
        response = client.post(
            "/subscriptions",
            data={
                "obj": "product",
                "id": "invalid_id",
                "action": "subscribe",
            },
            follow_redirects=True,
        )

        assert response.status_code == 400
        assert response.json["message"] == "product invalid_id does not exist"
        assert response.json["status"] == "error"


def test_subscribe_to_invalid_vendor_id_with_valid_uuid(login, client):

    with client:
        response = client.post(
            "/subscriptions",
            data={
                "obj": "vendor",
                "id": "38e0697e-b406-4724-80af-e39e354b291c",
                "action": "subscribe",
            },
            follow_redirects=True,
        )

        assert response.status_code == 400
        assert (
            response.json["message"]
            == "vendor 38e0697e-b406-4724-80af-e39e354b291c does not exist"
        )
        assert response.json["status"] == "error"


def test_subscribe_to_invalid_product_id_with_valid_uuid(login, client):

    with client:
        response = client.post(
            "/subscriptions",
            data={
                "obj": "product",
                "id": "38e0697e-b406-4724-80af-e39e354b291c",
                "action": "subscribe",
            },
            follow_redirects=True,
        )

        assert response.status_code == 400
        assert (
            response.json["message"]
            == "product 38e0697e-b406-4724-80af-e39e354b291c does not exist"
        )
        assert response.json["status"] == "error"



from sqlalchemy_utils import UUIDType
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.schema import UniqueConstraint

from opencve.extensions import db
from opencve.models import BaseModel, alerts_events


class UserTag(BaseModel):
    __tablename__ = "users_tags"

    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    color = db.Column(db.String(), nullable=False)

    # Relationships
    user_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("users.id"), nullable=False
    )
    user = db.relationship("User", back_populates="tags")

    __table_args__ = (
        UniqueConstraint("name", "user_id", name="ix_userstags_name_userid"),
    )

    def __repr__(self):
        return "<UserTag {}>".format(self.id)


class CveTag(BaseModel):
    __tablename__ = "cves_tags"

    tags = db.Column(JSONB)

    # Relationships
    user_id = db.Column(UUIDType(binary=False), db.ForeignKey("users.id"))
    user = db.relationship("User", back_populates="cve_tags")

    cve_id = db.Column(UUIDType(binary=False), db.ForeignKey("cves.id"))

    # Index
    __table_args__ = (db.Index("ix_cves_tags", tags, postgresql_using="gin"),)

    def __repr__(self):
        return "<CveTag {}>".format(self.id)



from nested_lookup import nested_lookup

from opencve.checks import BaseCheck
from opencve.commands.utils import CveUtil
from opencve.extensions import db
from opencve.models.products import Product
from opencve.models.vendors import Vendor
from opencve.utils import convert_cpes, flatten_vendors


class Cpes(BaseCheck):
    def execute(self):
        old = nested_lookup("cpe23Uri", self.cve_obj.json["configurations"])
        new = nested_lookup("cpe23Uri", self.cve_json["configurations"])

        payload = {
            "added": list(set(new) - set(old)),
            "removed": list(set(old) - set(new)),
        }

        # The CPEs list has been modified
        if payload["added"] or payload["removed"]:

            # Change the CVE's vendors attribute
            self.cve_obj.vendors = flatten_vendors(
                convert_cpes(self.cve_json["configurations"])
            )
            db.session.commit()

            # Create the vendors and products objects if they don't exist
            vendors_products = convert_cpes(payload["added"])

            for vendor, products in vendors_products.items():
                v_obj = Vendor.query.filter_by(name=vendor).first()

                # Create the vendor and associate it to the CVE
                if not v_obj:
                    v_obj = Vendor(name=vendor)
                    db.session.add(v_obj)
                    db.session.commit()

                # Do the same for its products
                for product in products:
                    p_obj = Product.query.filter_by(name=product, vendor=v_obj).first()
                    if not p_obj:
                        p_obj = Product(name=product, vendor=v_obj)
                        db.session.add(p_obj)
                        db.session.commit()

            # Create the event
            event = CveUtil.create_event(self.cve_obj, self.cve_json, "cpes", payload)
            return event

        return None



from io import BytesIO
from zipfile import ZipFile

import requests
import untangle

from opencve.commands import header, info, timed_operation
from opencve.extensions import db
from opencve.models import get_uuid
from opencve.models.cwe import Cwe

MITRE_CWE_URL = "https://cwe.mitre.org/data/xml/cwec_latest.xml.zip"


def run():
    """
    Import the CWE list.
    """
    header("Importing CWE list...")

    # Download the file
    with timed_operation("Downloading {}...".format(MITRE_CWE_URL)):
        resp = requests.get(MITRE_CWE_URL).content

    # Parse weaknesses
    with timed_operation("Parsing cwes..."):
        z = ZipFile(BytesIO(resp))
        raw = z.open(z.namelist()[0]).read()
        obj = untangle.parse(raw.decode("utf-8"))
        weaknesses = obj.Weakness_Catalog.Weaknesses.Weakness
        categories = obj.Weakness_Catalog.Categories.Category

    # Create the objects
    cwes = {}
    with timed_operation("Creating mappings..."):
        for c in weaknesses + categories:
            cwes[c["ID"]] = dict(
                id=get_uuid(),
                cwe_id=f"CWE-{c['ID']}",
                name=c["Name"],
                description=c.Description.cdata
                if hasattr(c, "Description")
                else c.Summary.cdata,
            )

    # Insert the objects in database
    with timed_operation("Inserting CWE..."):
        db.session.bulk_insert_mappings(Cwe, cwes.values())
        db.session.commit()

    info("{} CWE imported.".format(len(cwes)))
    del cwes



"""Add first_time value in event_types enum

Revision ID: 2132d05ea0e2
Revises: 4195eeb432e9
Create Date: 2021-12-19 22:59:41.176119

"""

# revision identifiers, used by Alembic.
revision = "2132d05ea0e2"
down_revision = "4195eeb432e9"

from alembic import op


def upgrade():
    with op.get_context().autocommit_block():
        op.execute("ALTER TYPE event_types ADD VALUE 'first_time';")


def downgrade():
    with op.get_context().autocommit_block():
        op.execute(
            "CREATE TYPE event_types_new AS ENUM ('new_cve', 'references', 'cpes', 'cvss', 'summary', 'cwes');"
        )
        op.execute(
            "ALTER TABLE events ALTER COLUMN type TYPE event_types_new USING (type::text::event_types_new);"
        )
        op.execute("DROP TYPE event_types;")
        op.execute("ALTER TYPE event_types_new RENAME TO event_types;")



from flask import request
from flask_restful import fields, marshal_with

from opencve.api.base import BaseResource
from opencve.api.cves import cves_fields
from opencve.api.fields import DatetimeField
from opencve.controllers.alerts import AlertController
from opencve.controllers.reports import ReportController
from opencve.models.users import User


alert_fields = {
    "id": fields.String(attribute="id"),
    "created_at": DatetimeField(),
    "cve": fields.String(attribute="cve.cve_id"),
    "details": fields.Raw(attribute="details"),
}

event_fields = {
    "cve": fields.String(attribute="cve.cve_id"),
    "type": fields.String(attribute="type.code"),
    "details": fields.Raw(attribute="details"),
}


class AlertListResource(BaseResource):
    @marshal_with(alert_fields)
    def get(self, link):
        report = ReportController.get({"public_link": link})
        return AlertController.list_items({"report_id": report.id})


class AlertResource(BaseResource):
    @marshal_with(event_fields)
    def get(self, link, id):
        report = ReportController.get({"public_link": link})
        alert = AlertController.get({"report_id": report.id, "id": id})
        return alert.events



from flask import current_app as app
from flask_paginate import Pagination

from opencve.controllers.base import BaseController
from opencve.controllers.vendors import VendorController
from opencve.models.products import Product


class ProductController(BaseController):
    model = Product
    order = [Product.name.asc()]
    per_page_param = "PRODUCTS_PER_PAGE"
    page_parameter = "product_page"
    schema = {
        "vendor": {"type": str},
        "search": {"type": str},
    }

    @classmethod
    def build_query(cls, args):
        if "vendor" in args:
            vendor = VendorController.get({"name": args.get("vendor")})
            query = cls.model.query.filter_by(vendor=vendor)
        else:
            query = cls.model.query

        # Search by term
        if args.get("search"):
            search = (
                args.get("search")
                .lower()
                .replace("%", "")
                .replace("_", "")
                .replace(" ", "_")
            )
            query = query.filter(Product.name.like("%{}%".format(search)))

        return query, {}

    @classmethod
    def get_pagination(cls, args, objects):
        return Pagination(
            product_page=args.get(cls.page_parameter),
            total=objects.total,
            per_page=app.config[cls.per_page_param],
            page_parameter=cls.page_parameter,
            record_name="objects",
            css_framework="bootstrap3",
        )

    @classmethod
    def get(cls, filters):
        vendor = VendorController.get({"name": filters.get("vendor")})
        return super(ProductController, cls).get(
            {"vendor_id": vendor.id, "name": filters.get("product")}
        )



"""Add tags tables

Revision ID: 4195eeb432e9
Revises: 33cd640e1112
Create Date: 2021-06-16 21:27:51.683602

"""

# revision identifiers, used by Alembic.
revision = "4195eeb432e9"
down_revision = "33cd640e1112"

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
from sqlalchemy_utils import UUIDType


def upgrade():
    op.create_table(
        "cves_tags",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("tags", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column("user_id", UUIDType(binary=False), nullable=True),
        sa.Column("cve_id", UUIDType(binary=False), nullable=True),
        sa.ForeignKeyConstraint(
            ["cve_id"],
            ["cves.id"],
        ),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["users.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        "ix_cves_tags", "cves_tags", ["tags"], unique=False, postgresql_using="gin"
    )
    op.create_index(
        op.f("ix_cves_tags_created_at"), "cves_tags", ["created_at"], unique=False
    )
    op.create_table(
        "users_tags",
        sa.Column("id", UUIDType(binary=False), nullable=False),
        sa.Column("created_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("updated_at", sa.DateTime(timezone=True), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("description", sa.String(), nullable=True),
        sa.Column("color", sa.String(), nullable=False),
        sa.Column(
            "user_id",
            UUIDType(binary=False),
            nullable=False,
        ),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["users.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name", "user_id", name="ix_userstags_name_userid"),
    )
    op.create_index(
        op.f("ix_users_tags_created_at"), "users_tags", ["created_at"], unique=False
    )


def downgrade():
    op.drop_index(op.f("ix_users_tags_created_at"), table_name="users_tags")
    op.drop_constraint("ix_userstags_name_userid", "users_tags")
    op.drop_table("users_tags")
    op.drop_index(op.f("ix_cves_tags_created_at"), table_name="cves_tags")
    op.drop_index("ix_cves_tags", table_name="cves_tags", postgresql_using="gin")
    op.drop_table("cves_tags")



import os
import configparser
from pathlib import Path


DEFAULT_CONFIG = Path(__file__).parent.resolve() / "default.cfg"
DEFAULT_WELCOME_FILES = Path(__file__).parent.resolve() / "templates/_welcome"
OPENCVE_HOME = os.environ.get("OPENCVE_HOME") or str(Path.home() / "opencve")
OPENCVE_CONFIG = os.environ.get("OPENCVE_CONFIG") or str(
    Path(OPENCVE_HOME) / "opencve.cfg"
)
OPENCVE_WELCOME_FILES = os.environ.get("OPENCVE_WELCOME_FILES") or str(
    Path(OPENCVE_HOME) / "welcome_html"
)

# Load the configuration
config = configparser.ConfigParser()

if Path(OPENCVE_CONFIG).exists():
    config.read(OPENCVE_CONFIG)
else:
    config.read(DEFAULT_CONFIG)

    # Generate a secret to avoid the following warning when init the config :
    # WARNING: Flask-User TokenManager: SECRET_KEY is shorter than 32 bytes.
    config.set("core", "secret_key", " " * 32)



from unittest.mock import patch

from opencve.extensions import db
from opencve.models.cve import Cve
from opencve.models.cwe import Cwe
from opencve.models.metas import Meta
from opencve.models.products import Product
from opencve.models.vendors import Vendor
from opencve.models.users import User
from opencve.models.products import Product
from opencve.models.vendors import Vendor
from opencve.models.tags import CveTag, UserTag
from opencve.models.users import User


def test_new_cve():
    cve = Cve(
        cve_id="CVE-2020-1234",
        json={"foo": "bar"},
        vendors=["vendor-1", "vendor-2", "product-1"],
        cwes=["CWE-1234"],
        summary="Example of summary",
        cvss2=5.0,
        cvss3=10.0,
        events=[],
        changes=[],
        alerts=[],
    )
    assert cve.cve_id == "CVE-2020-1234"
    assert cve.json == {"foo": "bar"}
    assert cve.vendors == ["vendor-1", "vendor-2", "product-1"]
    assert cve.cwes == ["CWE-1234"]
    assert cve.summary == "Example of summary"
    assert cve.cvss2 == 5.0
    assert cve.cvss3 == 10.0
    assert cve.events == []
    assert cve.changes == []
    assert cve.alerts == []


@patch("flask_login.utils._get_user")
def test_cve_tags(mock, create_cve, create_user):
    user = create_user()
    mock.return_value = user

    cve = create_cve("CVE-2018-18074")
    cve = Cve.query.first()
    assert not cve.tags

    user_tag = UserTag(name="tag1", description="foo", color="#fff")
    user.tags.append(user_tag)
    db.session.add(CveTag(user_id=user.id, cve_id=cve.id, tags=["tag1"]))
    db.session.commit()

    cve = Cve.query.first()
    assert len(cve.tags) == 1
    assert cve.tags[0].id == user_tag.id


def test_new_cwe():
    cwe = Cwe(
        cwe_id="CWE-79",
        name="Improper Neutralization...",
        description="The software does not...",
    )
    assert str(cwe) == "<Cwe CWE-79>"
    assert cwe.cwe_id == "CWE-79"
    assert cwe.short_id == "79"
    assert cwe.name == "Improper Neutralization..."
    assert cwe.description == "The software does not..."

    # Try an unusual CWE ID
    cwe.cwe_id = "NVD-CWE-noinfo"
    assert str(cwe) == "<Cwe NVD-CWE-noinfo>"
    assert cwe.cwe_id == "NVD-CWE-noinfo"
    assert not cwe.short_id


def test_new_meta():
    meta = Meta(
        name="foo",
        value="bar",
    )
    assert meta.name == "foo"
    assert meta.value == "bar"


def test_new_product():
    product = Product(name="Requests", vendor=Vendor(name="Python"))
    assert str(product) == "<Product Requests>"
    assert product.name == "Requests"
    assert product.vendor.name == "Python"


def test_product_with_users():
    product = Product(
        name="Requests",
        vendor=Vendor(name="Python"),
        users=[User(username="nicolas"), User(username="laurent")],
    )
    assert len(product.users) == 2
    assert [u.username for u in product.users] == ["nicolas", "laurent"]


def test_empty_vendor():
    vendor = Vendor(name="Python")
    assert str(vendor) == "<Vendor Python>"
    assert vendor.name == "Python"
    assert vendor.products == []
    assert vendor.users == []


def test_vendor_with_products():
    vendor = Vendor(
        name="Python",
        products=[
            Product(name="Requests"),
            Product(name="Celery"),
            Product(name="Virtualenv"),
        ],
    )

    assert len(vendor.products) == 3
    assert [p.name for p in vendor.products] == ["Requests", "Celery", "Virtualenv"]


def test_vendor_with_users():
    vendor = Vendor(
        name="Python", users=[User(username="nicolas"), User(username="laurent")]
    )
    assert len(vendor.users) == 2
    assert [u.username for u in vendor.users] == ["nicolas", "laurent"]
